@@ -16,21 +16,22 @@
 """Authentication token retrieval."""
 
 from bzrlib.config import AuthenticationConfig
+from bzrlib.trace import mutter
 from bzrlib.ui import ui_factory
-import svn.core
-from svn.core import (svn_auth_cred_username_t, 
-                      svn_auth_cred_simple_t,
-                      svn_auth_cred_ssl_client_cert_t,
-                      svn_auth_cred_ssl_client_cert_pw_t,
-                      svn_auth_cred_ssl_server_trust_t,
-                      svn_auth_get_username_prompt_provider,
-                      svn_auth_get_simple_prompt_provider,
-                      svn_auth_get_ssl_server_trust_prompt_provider,
-                      svn_auth_get_ssl_client_cert_pw_prompt_provider,
-                      svn_auth_open)
+
+from bzrlib.plugins.svn import ra
+
 import urlparse
 import urllib
 
+AUTH_PARAM_DEFAULT_USERNAME = 'svn:auth:username'
+AUTH_PARAM_DEFAULT_PASSWORD = 'svn:auth:password'
+
+SSL_NOTYETVALID = 0x00000001
+SSL_EXPIRED     = 0x00000002
+SSL_CNMISMATCH  = 0x00000004
+SSL_UNKNOWNCA   = 0x00000008
+SSL_OTHER       = 0x40000000
 
 class SubversionAuthenticationConfig(AuthenticationConfig):
     """Simple extended version of AuthenticationConfig that can provide 
@@ -43,41 +44,35 @@ class SubversionAuthenticationConfig(AuthenticationConfig):
         self.port = port
         self.path = path
        
-    def get_svn_username(self, realm, may_save, pool=None):
+    def get_svn_username(self, realm, may_save):
         """Look up a Subversion user name in the Bazaar authentication cache.
 
         :param realm: Authentication realm (optional)
         :param may_save: Whether or not the username should be saved.
-        :param pool: Allocation pool, is ignored.
-        :param default: Assumed username
         """
-        username_cred = svn_auth_cred_username_t()
-        username_cred.username = self.get_user(self.scheme, 
-                host=self.host, path=self.path, 
-                realm=realm)
-        username_cred.may_save = False
-        return username_cred
+        mutter("Obtaining username for SVN connection")
+        username = self.get_user(self.scheme, host=self.host, path=self.path, realm=realm)
+        if username is None:
+            return None
+        return (username, False)
 
-    def get_svn_simple(self, realm, username, may_save, pool):
+    def get_svn_simple(self, realm, username, may_save):
         """Look up a Subversion user name+password combination in the Bazaar 
         authentication cache.
 
         :param realm: Authentication realm (optional)
         :param username: Username, if it is already known, or None.
         :param may_save: Whether or not the username should be saved.
-        :param pool: Allocation pool, is ignored.
         """
-        simple_cred = svn_auth_cred_simple_t()
-        simple_cred.username = self.get_user(self.scheme, 
+        mutter("Obtaining username and password for SVN connection")
+        username = self.get_user(self.scheme, 
                 host=self.host, path=self.path, realm=realm) or username
-        simple_cred.password = self.get_password(self.scheme, host=self.host, 
-            path=self.path, user=simple_cred.username, 
-            realm=realm, prompt="%s %s password" % (realm, simple_cred.username))
-        simple_cred.may_save = False
-        return simple_cred
+        password = self.get_password(self.scheme, host=self.host, 
+            path=self.path, user=username, 
+            realm=realm, prompt="%s %s password" % (realm, username))
+        return (username, password, False)
 
-    def get_svn_ssl_server_trust(self, realm, failures, cert_info, may_save, 
-                                     pool):
+    def get_svn_ssl_server_trust(self, realm, failures, cert_info, may_save):
         """Return a Subversion auth provider that verifies SSL server trust.
 
         :param realm: Realm name (optional)
@@ -85,21 +80,20 @@ class SubversionAuthenticationConfig(AuthenticationConfig):
         :param cert_info: Certificate information
         :param may_save: Whether this information may be stored.
         """
-        ssl_server_trust = svn_auth_cred_ssl_server_trust_t()
+        mutter("Verifying SSL server: %s", realm)
         credentials = self.get_credentials(self.scheme, host=self.host)
         if (credentials is not None and 
             credentials.has_key("verify_certificates") and 
             credentials["verify_certificates"] == False):
-            ssl_server_trust.accepted_failures = (
-                    svn.core.SVN_AUTH_SSL_NOTYETVALID + 
-                    svn.core.SVN_AUTH_SSL_EXPIRED +
-                    svn.core.SVN_AUTH_SSL_CNMISMATCH +
-                    svn.core.SVN_AUTH_SSL_UNKNOWNCA +
-                    svn.core.SVN_AUTH_SSL_OTHER)
+            accepted_failures = (
+                    SSL_NOTYETVALID + 
+                    SSL_EXPIRED +
+                    SSL_CNMISMATCH +
+                    SSL_UNKNOWNCA +
+                    SSL_OTHER)
         else:
-            ssl_server_trust.accepted_failures = 0
-        ssl_server_trust.may_save = False
-        return ssl_server_trust
+            accepted_failures = 0
+        return (accepted_failures, False)
 
     def get_svn_username_prompt_provider(self, retries):
         """Return a Subversion auth provider for retrieving the username, as 
@@ -107,7 +101,7 @@ class SubversionAuthenticationConfig(AuthenticationConfig):
         
         :param retries: Number of allowed retries.
         """
-        return svn_auth_get_username_prompt_provider(self.get_svn_username, 
+        return ra.get_username_prompt_provider(self.get_svn_username, 
                                                      retries)
 
     def get_svn_simple_prompt_provider(self, retries):
@@ -116,12 +110,13 @@ class SubversionAuthenticationConfig(AuthenticationConfig):
         
         :param retries: Number of allowed retries.
         """
-        return svn_auth_get_simple_prompt_provider(self.get_svn_simple, retries)
+        return ra.get_simple_prompt_provider(self.get_svn_simple, retries)
 
     def get_svn_ssl_server_trust_prompt_provider(self):
         """Return a Subversion auth provider for checking 
         whether a SSL server is trusted."""
-        return svn_auth_get_ssl_server_trust_prompt_provider(self.get_svn_ssl_server_trust)
+        return ra.get_ssl_server_trust_prompt_provider(
+            self.get_svn_ssl_server_trust)
 
     def get_svn_auth_providers(self):
         """Return a list of auth providers for this authentication file.
@@ -130,40 +125,39 @@ class SubversionAuthenticationConfig(AuthenticationConfig):
                 self.get_svn_simple_prompt_provider(1),
                 self.get_svn_ssl_server_trust_prompt_provider()]
 
-def get_ssl_client_cert_pw(realm, may_save, pool):
+
+def get_ssl_client_cert_pw(realm, may_save):
     """Simple SSL client certificate password prompter.
 
     :param realm: Realm, optional.
     :param may_save: Whether the password can be cached.
     """
-    ssl_cred_pw = svn_auth_cred_ssl_client_cert_pw_t()
-    ssl_cred_pw.password = ui_factory.get_password(
+    password = ui_factory.get_password(
             "Please enter password for client certificate[realm=%s]" % realm)
-    ssl_cred_pw.may_save = False
-    return ssl_cred_pw
+    return (password, False)
 
 
 def get_ssl_client_cert_pw_provider(tries):
-    return svn_auth_get_ssl_client_cert_pw_prompt_provider(
+    return ra.get_ssl_client_cert_pw_prompt_provider(
                 get_ssl_client_cert_pw, tries)
 
 
 def get_stock_svn_providers():
-    providers = [svn.client.get_simple_provider(),
-            svn.client.get_username_provider(),
-            svn.client.get_ssl_client_cert_file_provider(),
-            svn.client.get_ssl_client_cert_pw_file_provider(),
-            svn.client.get_ssl_server_trust_file_provider(),
+    providers = [ra.get_simple_provider(),
+            ra.get_username_provider(),
+            ra.get_ssl_client_cert_file_provider(),
+            ra.get_ssl_client_cert_pw_file_provider(),
+            ra.get_ssl_server_trust_file_provider(),
             ]
 
-    if hasattr(svn.client, 'get_windows_simple_provider'):
-        providers.append(svn.client.get_windows_simple_provider())
+    if getattr(ra, 'get_windows_simple_provider', None):
+        providers.append(ra.get_windows_simple_provider())
 
-    if hasattr(svn.client, 'get_keychain_simple_provider'):
-        providers.append(svn.client.get_keychain_simple_provider())
+    if getattr(ra, 'get_keychain_simple_provider', None):
+        providers.append(ra.get_keychain_simple_provider())
 
-    if hasattr(svn.client, 'get_windows_ssl_server_trust_provider'):
-        providers.append(svn.client.get_windows_ssl_server_trust_provider())
+    if getattr(ra, 'get_windows_ssl_server_trust_provider', None):
+        providers.append(ra.get_windows_ssl_server_trust_provider())
 
     return providers
 
@@ -179,19 +173,16 @@ def create_auth_baton(url):
 
     # Specify Subversion providers first, because they use file data
     # rather than prompting the user.
-    providers = get_stock_svn_providers()
+    providers = []
+    providers += get_stock_svn_providers()
+    providers += auth_config.get_svn_auth_providers()
+    providers += [get_ssl_client_cert_pw_provider(1)]
 
-    if svn.core.SVN_VER_MAJOR == 1 and svn.core.SVN_VER_MINOR >= 5:
-        providers += auth_config.get_svn_auth_providers()
-        providers += [get_ssl_client_cert_pw_provider(1)]
-
-    auth_baton = svn.core.svn_auth_open(providers)
+    auth_baton = ra.Auth(providers)
     if creds is not None:
-        (auth_baton.user, auth_baton.password) = urllib.splitpasswd(creds)
-        if auth_baton.user is not None:
-            svn.core.svn_auth_set_parameter(auth_baton, 
-                svn.core.SVN_AUTH_PARAM_DEFAULT_USERNAME, auth_baton.user)
-        if auth_baton.password is not None:
-            svn.core.svn_auth_set_parameter(auth_baton, 
-                svn.core.SVN_AUTH_PARAM_DEFAULT_PASSWORD, auth_baton.password)
+        (user, password) = urllib.splitpasswd(creds)
+        if user is not None:
+            auth_baton.set_parameter(AUTH_PARAM_DEFAULT_USERNAME, user)
+        if password is not None:
+            auth_baton.set_parameter(AUTH_PARAM_DEFAULT_PASSWORD, password)
     return auth_baton
