@@ -15,25 +15,25 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 """Handles branch-specific operations."""
 
-from bzrlib import ui
+from bzrlib import ui, urlutils
 from bzrlib.branch import Branch, BranchFormat, BranchCheckResult, PullResult
 from bzrlib.bzrdir import BzrDir
 from bzrlib.errors import (NoSuchFile, DivergedBranches, NoSuchRevision, 
-                           NotBranchError)
-from bzrlib.inventory import (Inventory)
-from bzrlib.revision import is_null, ensure_null, NULL_REVISION
+                           NotBranchError, UnstackableBranchFormat)
+from bzrlib.revision import is_null, ensure_null
 from bzrlib.workingtree import WorkingTree
 
-import svn.client, svn.core
-from svn.core import SubversionException
-
-from commit import push
-from config import BranchConfig
-from errors import NotSvnBranchPath
-from format import get_rich_root_format
-from repository import SvnRepository
-from transport import bzr_to_svn_url, create_svn_client
+from bzrlib.plugins.svn import core, wc
+from bzrlib.plugins.svn.commit import push, push_ancestors
+from bzrlib.plugins.svn.config import BranchConfig
+from bzrlib.plugins.svn.core import SubversionException
+from bzrlib.plugins.svn.errors import NotSvnBranchPath, ERR_FS_NO_SUCH_REVISION
+from bzrlib.plugins.svn.format import get_rich_root_format
+from bzrlib.plugins.svn.repository import SvnRepository
+from bzrlib.plugins.svn.tags import SubversionTags
+from bzrlib.plugins.svn.transport import bzr_to_svn_url
 
+import os
 
 class FakeControlFiles(object):
     """Dummy implementation of ControlFiles.
@@ -52,7 +52,7 @@ class FakeControlFiles(object):
 
 class SvnBranch(Branch):
     """Maps to a Branch in a Subversion repository """
-    def __init__(self, base, repository, branch_path):
+    def __init__(self, repository, branch_path, _skip_check=False):
         """Instantiate a new SvnBranch.
 
         :param repos: SvnRepository this branch is part of.
@@ -61,32 +61,38 @@ class SvnBranch(Branch):
         :param revnum: Subversion revision number of the branch to 
             look at; none for latest.
         """
-        super(SvnBranch, self).__init__()
         self.repository = repository
+        super(SvnBranch, self).__init__()
         assert isinstance(self.repository, SvnRepository)
         self.control_files = FakeControlFiles()
-        self.base = base.rstrip("/")
         self._format = SvnBranchFormat()
         self._lock_mode = None
         self._lock_count = 0
         self.mapping = self.repository.get_mapping()
+        self.layout = self.repository.get_layout()
         self._branch_path = branch_path.strip("/")
+        self.base = urlutils.join(self.repository.base, 
+                        self._branch_path).rstrip("/")
         self._revmeta_cache = None
         assert isinstance(self._branch_path, str)
-        try:
-            revnum = self.get_revnum()
-            if revnum is None:
-                raise NotBranchError(self.base)
-            if self.repository.transport.check_path(branch_path.strip("/"), 
-                revnum) != svn.core.svn_node_dir:
-                raise NotBranchError(self.base)
-        except SubversionException, (_, num):
-            if num == svn.core.SVN_ERR_FS_NO_SUCH_REVISION:
-                raise NotBranchError(self.base)
-            raise
-        if not self.mapping.is_branch(branch_path):
+        if not _skip_check:
+            try:
+                revnum = self.get_revnum()
+                if self.repository.transport.check_path(self._branch_path, 
+                    revnum) != core.NODE_DIR:
+                    raise NotBranchError(self.base)
+            except SubversionException, (_, num):
+                if num == ERR_FS_NO_SUCH_REVISION:
+                    raise NotBranchError(self.base)
+                raise
+        (type, self.project, _, ip) = self.layout.parse(branch_path)
+        # FIXME: Don't allow tag here
+        if type not in ('branch', 'tag') or ip != '':
             raise NotSvnBranchPath(branch_path, mapping=self.mapping)
 
+    def _make_tags(self):
+        return SubversionTags(self)
+
     def set_branch_path(self, branch_path):
         """Change the branch path for this branch.
 
@@ -103,7 +109,8 @@ class SvnBranch(Branch):
 
         :param relpath: path from the repository root.
         """
-        assert relpath.startswith(self.get_branch_path())
+        assert relpath.startswith(self.get_branch_path()), \
+                "expected %s prefix, got %s" % (self.get_branch_path(), relpath)
         return relpath[len(self.get_branch_path()):].strip("/")
 
     def get_branch_path(self, revnum=None):
@@ -114,8 +121,13 @@ class SvnBranch(Branch):
         if revnum is None:
             return self._branch_path
 
-        # TODO: Use revnum - this branch may have been moved in the past 
-        return self._branch_path
+        if revnum == self.get_revnum():
+            return self._branch_path
+
+        # Use revnum - this branch may have been moved in the past 
+        return self.repository.transport.get_locations(
+                    self._branch_path, self.get_revnum(), 
+                    [revnum])[revnum].strip("/")
 
     def get_revnum(self):
         """Obtain the Subversion revision number this branch was 
@@ -126,7 +138,10 @@ class SvnBranch(Branch):
         if self._lock_mode == 'r' and self._cached_revnum:
             return self._cached_revnum
         latest_revnum = self.repository.get_latest_revnum()
-        self._cached_revnum = self.repository._log.find_latest_change(self.get_branch_path(), latest_revnum)
+        self._cached_revnum = self.repository._log.find_latest_change(
+            self.get_branch_path(), latest_revnum)
+        if self._cached_revnum is None:
+            raise NotBranchError(self.base)
         return self._cached_revnum
 
     def check(self):
@@ -136,7 +151,8 @@ class SvnBranch(Branch):
         """
         return BranchCheckResult(self)
 
-    def _create_heavyweight_checkout(self, to_location, revision_id=None, hardlink=False):
+    def _create_heavyweight_checkout(self, to_location, revision_id=None, 
+                                     hardlink=False):
         """Create a new heavyweight checkout of this branch.
 
         :param to_location: URL of location to create the new checkout in.
@@ -162,7 +178,7 @@ class SvnBranch(Branch):
         :raises NoSuchRevision: If the revision id was not found.
         """
         (bp, revnum, mapping) = self.repository.lookup_revision_id(revid, 
-                                         ancestry=(self.get_branch_path(), self.get_revnum()))
+            ancestry=(self.get_branch_path(), self.get_revnum()))
         assert bp.strip("/") == self.get_branch_path(revnum).strip("/"), \
                 "Got %r, expected %r" % (bp, self.get_branch_path(revnum))
         return revnum
@@ -174,23 +190,28 @@ class SvnBranch(Branch):
         :param revision_id: Tip of the checkout.
         :return: WorkingTree object of the checkout.
         """
-        peg_rev = svn.core.svn_opt_revision_t()
-        peg_rev.kind = svn.core.svn_opt_revision_head
-
-        rev = svn.core.svn_opt_revision_t()
-        if revision_id is None:
-            rev.kind = svn.core.svn_opt_revision_head
-        else:
+        from bzrlib.plugins.svn.workingtree import update_wc
+        if revision_id is not None:
             revnum = self.lookup_revision_id(revision_id)
-            rev.kind = svn.core.svn_opt_revision_number
-            rev.value.number = revnum
+        else:
+            revnum = self.get_revnum()
 
         svn_url = bzr_to_svn_url(self.base)
-        client_ctx = create_svn_client(svn_url)
-        svn.client.checkout(svn_url, to_location, rev, 
-                            True, client_ctx)
-
-        return WorkingTree.open(to_location)
+        os.mkdir(to_location)
+        wc.ensure_adm(to_location, self.repository.uuid, svn_url,
+                      bzr_to_svn_url(self.repository.base), revnum)
+        adm = wc.WorkingCopy(None, to_location, write_lock=True)
+        try:
+            conn = self.repository.transport.connections.get(svn_url)
+            try:
+                update_wc(adm, to_location, conn, revnum)
+            finally:
+                if not conn.busy:
+                    self.repository.transport.add_connection(conn)
+        finally:
+            adm.close()
+        wt = WorkingTree.open(to_location)
+        return wt
 
     def create_checkout(self, to_location, revision_id=None, lightweight=False,
                         accelerator_tree=None, hardlink=False):
@@ -198,13 +219,19 @@ class SvnBranch(Branch):
         if lightweight:
             return self._create_lightweight_checkout(to_location, revision_id)
         else:
-            return self._create_heavyweight_checkout(to_location, revision_id, hardlink=hardlink)
+            return self._create_heavyweight_checkout(to_location, revision_id, 
+                                                     hardlink=hardlink)
 
     def generate_revision_id(self, revnum):
         """Generate a new revision id for a revision on this branch."""
         assert isinstance(revnum, int)
-        return self.repository.generate_revision_id(
+        try:
+            return self.repository.generate_revision_id(
                 revnum, self.get_branch_path(revnum), self.mapping)
+        except SubversionException, (_, num):
+            if num == ERR_FS_NO_SUCH_REVISION:
+                raise NoSuchRevision(self, revnum)
+            raise
 
     def get_config(self):
         return BranchConfig(self)
@@ -216,18 +243,56 @@ class SvnBranch(Branch):
         """
         bp = self._branch_path.strip("/")
         if self._branch_path == "":
-            return None
+            return self.base.split("/")[-1]
         return bp
 
     nick = property(_get_nick)
 
     def set_revision_history(self, rev_history):
         """See Branch.set_revision_history()."""
-        raise NotImplementedError(self.set_revision_history)
+        if (rev_history == [] or 
+            not self.repository.has_revision(rev_history[-1])):
+            raise NotImplementedError("set_revision_history can't add ghosts")
+        push(self.repository.get_graph(), 
+             self, self.repository, rev_history[-1])
+        self._clear_cached_state()
 
     def set_last_revision_info(self, revno, revid):
         """See Branch.set_last_revision_info()."""
 
+    def mainline_missing_revisions(self, other, stop_revision):
+        """Find the revisions missing on the mainline.
+        
+        :param other: Other branch to retrieve revisions from.
+        :param stop_revision: Revision to stop fetching at.
+        """
+        missing = []
+        lastrevid = self.last_revision()
+        for revid in other.repository.iter_reverse_revision_history(stop_revision):
+            if lastrevid == revid:
+                missing.reverse()
+                return missing
+            missing.append(revid)
+        return None
+
+    def otherline_missing_revisions(self, other, stop_revision, overwrite=False):
+        """Find the revisions missing on the mainline.
+        
+        :param other: Other branch to retrieve revisions from.
+        :param stop_revision: Revision to stop fetching at.
+        :param overwrite: Whether or not the existing data should be overwritten
+        """
+        missing = []
+        for revid in other.repository.iter_reverse_revision_history(stop_revision):
+            if self.repository.has_revision(revid):
+                missing.reverse()
+                return missing
+            missing.append(revid)
+        if not overwrite:
+            return None
+        else:
+            return missing
+ 
     def last_revision_info(self):
         """See Branch.last_revision_info()."""
         last_revid = self.last_revision()
@@ -263,7 +328,7 @@ class SvnBranch(Branch):
         if self._revmeta_cache is None:
             pb = ui.ui_factory.nested_progress_bar()
             try:
-                self._revmeta_cache = list(self.repository.iter_reverse_branch_changes(self.get_branch_path(), self.get_revnum(), self.mapping, pb=pb))
+                self._revmeta_cache = list(self.repository.iter_reverse_branch_changes(self.get_branch_path(), self.get_revnum(), to_revnum=0, mapping=self.mapping, pb=pb))
             finally:
                 pb.finished()
         return self._revmeta_cache
@@ -288,7 +353,7 @@ class SvnBranch(Branch):
         return self.generate_revision_id(self.get_revnum())
 
     def pull(self, source, overwrite=False, stop_revision=None, 
-             _hook_master=None, run_hooks=True):
+             _hook_master=None, run_hooks=True, _push_merged=None):
         """See Branch.pull()."""
         result = PullResult()
         result.source_branch = source
@@ -297,13 +362,9 @@ class SvnBranch(Branch):
         source.lock_read()
         try:
             (result.old_revno, result.old_revid) = self.last_revision_info()
-            try:
-                self.update_revisions(source, stop_revision)
-            except DivergedBranches:
-                if overwrite:
-                    raise NotImplementedError('overwrite not supported for '
-                                              'Subversion branches')
-                raise
+            self.update_revisions(source, stop_revision, overwrite, 
+                                  _push_merged=_push_merged)
+            result.tag_conflicts = source.tags.merge_to(self.tags, overwrite)
             (result.new_revno, result.new_revid) = self.last_revision_info()
             return result
         finally:
@@ -340,27 +401,45 @@ class SvnBranch(Branch):
             revno = self.revision_id_to_revno(revision_id)
         destination.set_last_revision_info(revno, revision_id)
 
-    def update_revisions(self, other, stop_revision=None):
+    def update_revisions(self, other, stop_revision=None, overwrite=False, 
+                         graph=None, _push_merged=False):
         """See Branch.update_revisions()."""
         if stop_revision is None:
             stop_revision = ensure_null(other.last_revision())
         if (self.last_revision() == stop_revision or
             self.last_revision() == other.last_revision()):
             return
-        if not other.repository.get_graph().is_ancestor(self.last_revision(), 
+        if graph is None:
+            graph = self.repository.get_graph()
+        other_graph = other.repository.get_graph()
+        if not other_graph.is_ancestor(self.last_revision(), 
                                                         stop_revision):
-            if self.repository.get_graph().is_ancestor(stop_revision, 
-                                                       self.last_revision()):
+            if graph.is_ancestor(stop_revision, self.last_revision()):
                 return
+            if not overwrite:
+                raise DivergedBranches(self, other)
+        todo = self.mainline_missing_revisions(other, stop_revision)
+        if todo is None:
+            # Not possible to add cleanly onto mainline, perhaps need a replace operation
+            todo = self.otherline_missing_revisions(other, stop_revision, overwrite)
+        if todo is None:
             raise DivergedBranches(self, other)
-        todo = self.repository.lhs_missing_revisions(other.revision_history(), 
-                                                     stop_revision)
+        if _push_merged is None:
+            _push_merged = self.layout.push_merged_revisions(self.project)
+        self._push_missing_revisions(graph, other, other_graph, todo, 
+                                     _push_merged)
+
+    def _push_missing_revisions(self, my_graph, other, other_graph, todo, 
+                                push_merged=False):
         pb = ui.ui_factory.nested_progress_bar()
         try:
             for revid in todo:
                 pb.update("pushing revisions", todo.index(revid), 
                           len(todo))
-                push(self, other, revid)
+                if push_merged:
+                    parent_revids = other_graph.get_parent_map([revid])[revid]
+                    push_ancestors(self.repository, other.repository, self.layout, self.project, parent_revids, other_graph)
+                push(my_graph, self, other.repository, revid)
                 self._clear_cached_state()
         finally:
             pb.finished()
@@ -395,7 +474,7 @@ class SvnBranch(Branch):
         self.repository.unlock()
 
     def _clear_cached_state(self):
-        super(SvnBranch,self)._clear_cached_state()
+        super(SvnBranch, self)._clear_cached_state()
         self._cached_revnum = None
         self._revmeta_cache = None
 
@@ -406,13 +485,6 @@ class SvnBranch(Branch):
     def set_parent(self, url):
         """See Branch.set_parent()."""
 
-    def append_revision(self, *revision_ids):
-        """See Branch.append_revision()."""
-        self._clear_cached_state()
-        #raise NotImplementedError(self.append_revision)
-        #FIXME: Make sure the appended revision is already 
-        # part of the revision history
-
     def get_physical_lock_status(self):
         """See Branch.get_physical_lock_status()."""
         return False
@@ -421,11 +493,18 @@ class SvnBranch(Branch):
         """See Branch.sprout()."""
         result = to_bzrdir.create_branch()
         self.copy_content_into(result, revision_id=revision_id)
+        result.set_parent(self.bzrdir.root_transport.base)
         return result
 
+    def get_stacked_on_url(self):
+        raise UnstackableBranchFormat(self._format, self.base)
+
     def __str__(self):
         return '%s(%r)' % (self.__class__.__name__, self.base)
 
+    def supports_tags(self):
+        return self._format.supports_tags()
+
     __repr__ = __str__
 
 
@@ -453,3 +532,5 @@ class SvnBranchFormat(BranchFormat):
         """See BranchFormat.initialize()."""
         raise NotImplementedError(self.initialize)
 
+    def supports_tags(self):
+        return True
