@@ -17,10 +17,10 @@
 """Branch property access and caching."""
 
 from bzrlib.errors import NoSuchRevision
-from bzrlib.trace import mutter
 
-from svn.core import SubversionException
-import svn.core
+from bzrlib.plugins.svn import logwalker
+from bzrlib.plugins.svn.core import SubversionException
+from bzrlib.plugins.svn.errors import ERR_FS_NO_SUCH_REVISION
 
 
 class PathPropertyProvider(object):
@@ -41,7 +41,7 @@ class PathPropertyProvider(object):
             (_, _, props) = self.log._transport.get_dir(path, 
                 revnum)
         except SubversionException, (_, num):
-            if num == svn.core.SVN_ERR_FS_NO_SUCH_REVISION:
+            if num == ERR_FS_NO_SUCH_REVISION:
                 raise NoSuchRevision(self, revnum)
             raise
 
@@ -58,7 +58,10 @@ class PathPropertyProvider(object):
         """
         assert isinstance(revnum, int)
         assert isinstance(path, str)
-        if not path in self.log.get_revision_paths(revnum):
+        return logwalker.lazy_dict({}, self._real_get_changed_properties, path, revnum)
+
+    def _real_get_changed_properties(self, path, revnum):
+        if self.log.get_change(path, revnum) is None:
             return {}
         current = self.get_properties(path, revnum)
         if current == {}:
@@ -67,8 +70,8 @@ class PathPropertyProvider(object):
         if prev_path is None and prev_revnum == -1:
             previous = {}
         else:
-            previous = self.get_properties(prev_path.encode("utf-8"), 
-                                           prev_revnum)
+            assert isinstance(prev_path, str)
+            previous = self.get_properties(prev_path, prev_revnum)
         ret = {}
         for key, val in current.items():
             if previous.get(key) != val:
