@@ -17,26 +17,36 @@
 
 from bzrlib import ui
 from bzrlib.errors import NotBranchError, RevisionNotPresent
-from bzrlib.knit import make_file_knit
+from bzrlib.knit import make_file_factory
 from bzrlib.revision import NULL_REVISION
 from bzrlib.trace import mutter
+from bzrlib.versionedfile import ConstantMapper
 
 import urllib
 
-import changes
-from bzrlib.plugins.svn.mapping import escape_svn_path
+from bzrlib.plugins.svn import changes
 
-def get_local_changes(paths, branch, mapping, generate_revid, 
+def get_local_changes(paths, branch, mapping, layout, generate_revid, 
                       get_children=None):
+    """Obtain all of the changes relative to a particular path
+    (usually a branch path).
+
+    :param paths: Changes
+    :param branch: Path under which to select changes
+    :param mapping: Mapping to use to determine what are valid branch paths
+    :param layout: Layout to use 
+    :param generate_revid: Function for generating revision id from svn revnum
+    :param get_children: Function for obtaining the children of a path
+    """
     new_paths = {}
-    for p in sorted(paths.keys()):
+    for p in sorted(paths.keys(), reverse=False):
         if not changes.path_is_child(branch, p):
             continue
         data = paths[p]
-        new_p = mapping.scheme.unprefix(p)[1]
+        new_p = layout.parse(p)[3]
         if data[1] is not None:
             try:
-                (cbp, crp) = mapping.scheme.unprefix(data[1])
+                (pt, proj, cbp, crp) = layout.parse(data[1])
 
                 # Branch copy
                 if (crp == "" and new_p == ""):
@@ -49,7 +59,7 @@ def get_local_changes(paths, branch, mapping, generate_revid,
                 # Make it look like the files were added in this revision
                 if get_children is not None:
                     for c in get_children(data[1], data[2]):
-                        mutter('oops: %r child %r' % (data[1], c))
+                        mutter('oops: %r child %r', data[1], c)
                         new_paths[(new_p+"/"+c[len(data[1]):].strip("/")).strip("/")] = (data[0], None, -1)
                 data = (data[0], None, -1)
 
@@ -57,31 +67,35 @@ def get_local_changes(paths, branch, mapping, generate_revid,
     return new_paths
 
 
-FILEIDMAP_VERSION = 1
+FILEIDMAP_VERSION = 2
 
 def simple_apply_changes(new_file_id, changes, find_children=None):
-    """Simple function that can apply file id changes.
+    """Simple function that generates a dictionary with file id changes.
     
     Does not track renames. """
     map = {}
-    for p in sorted(changes.keys()):
+    for p in sorted(changes.keys(), reverse=False):
         data = changes[p]
 
+        inv_p = p.decode("utf-8")
+        if data[0] in ('D', 'R'):
+            if not inv_p in map:
+                map[inv_p] = None
         if data[0] in ('A', 'R'):
-            inv_p = p.decode("utf-8")
             map[inv_p] = new_file_id(inv_p)
 
             if data[1] is not None:
-                mutter('%r copied from %r:%s' % (inv_p, data[1], data[2]))
+                mutter('%r copied from %r:%s', inv_p, data[1], data[2])
                 if find_children is not None:
                     for c in find_children(data[1], data[2]):
                         inv_c = c.decode("utf-8")
                         path = c.replace(data[1].decode("utf-8"), inv_p+"/", 1).replace(u"//", u"/")
                         map[path] = new_file_id(path)
-                        mutter('added mapping %r -> %r' % (path, map[path]))
+                        mutter('added mapping %r -> %r', path, map[path])
 
     return map
 
+
 class FileIdMap(object):
     """File id store. 
 
@@ -93,18 +107,23 @@ class FileIdMap(object):
         self.apply_changes_fn = apply_changes_fn
         self.repos = repos
 
-    def apply_changes(self, uuid, revnum, branch, global_changes, 
-                      renames, mapping, find_children=None):
+    def _use_text_revids(self, mapping, revmeta, map):
+        text_revids = mapping.import_text_parents(revmeta.revprops, revmeta.fileprops).items()
+        for path, revid in text_revids:
+            assert path in map
+            map[path] = (map[path][0], revid)
+
+    def apply_changes(self, revmeta, mapping, find_children=None):
         """Change file id map to incorporate specified changes.
 
-        :param uuid: UUID of repository changes happen in
-        :param revnum: Revno for revision in which changes happened
-        :param branch: Branch path where changes happened
-        :param global_changes: Dict with global changes that happened
+        :param revmeta: RevisionMetadata object for revision with changes
         :param renames: List of renames (known file ids for particular paths)
         :param mapping: Mapping
         """
-        changes = get_local_changes(global_changes, branch, mapping,
+        renames = mapping.import_fileid_map(revmeta.revprops, revmeta.fileprops)
+        assert revmeta.paths is not None
+        changes = get_local_changes(revmeta.paths, revmeta.branch_path, mapping,
+                    self.repos.get_layout(),
                     self.repos.generate_revision_id, find_children)
         if find_children is not None:
             def get_children(path, revid):
@@ -115,7 +134,7 @@ class FileIdMap(object):
             get_children = None
 
         def new_file_id(x):
-            return mapping.generate_file_id(uuid, revnum, branch, x)
+            return mapping.generate_file_id(revmeta.uuid, revmeta.revnum, revmeta.branch_path, x)
          
         idmap = self.apply_changes_fn(new_file_id, changes, get_children)
         idmap.update(renames)
@@ -137,79 +156,92 @@ class FileIdMap(object):
             map = {}
 
         # No history -> empty map
-        for revmeta in self.repos.iter_reverse_branch_changes(branch, revnum, mapping):
+        for revmeta in self.repos.iter_reverse_branch_changes(branch, revnum, to_revnum=0, mapping=mapping):
             revid = revmeta.get_revision_id(mapping)
             todo.append(revmeta)
    
         pb = ui.ui_factory.nested_progress_bar()
 
         try:
-            i = 1
-            for revmeta in reversed(todo):
-                revid = revmeta.get_revision_id(mapping)
-                expensive = False
-                def log_find_children(path, revnum):
-                    expensive = True
-                    return self.repos._log.find_children(path, revnum)
-
-                (idmap, changes) = self.apply_changes(self.repos.uuid, 
-                        revmeta.revnum, revmeta.branch_path, revmeta.paths,
-                        mapping.import_fileid_map(revmeta.revprops, revmeta.fileprops), 
-                        mapping, log_find_children)
+            for i, revmeta in enumerate(reversed(todo)):
                 pb.update('generating file id map', i, len(todo))
+                revid = revmeta.get_revision_id(mapping)
+                (idmap, changes) = self.apply_changes(revmeta, 
+                        mapping, self.repos._log.find_children)
+                self.update_map(map, revid, idmap, changes)
+                self._use_text_revids(mapping, revmeta, map)
 
                 parent_revs = next_parent_revs
 
-                self.update_map(map, revid, idmap, changes)
-                       
                 next_parent_revs = [revid]
-                i += 1
         finally:
             pb.finished()
         return map
 
-    def update_map(self, map, revid, idmap, changes):
+    def update_map(self, map, revid, delta, changes):
+        """Update a file id map.
+
+        :param map: Existing file id map.
+        :param revid: Revision id of the id map
+        :param delta: Id map for just the delta
+        :param changes: Changes in revid.
+        """
         for p in changes:
-            if changes[p][0] == 'M' and not idmap.has_key(p):
-                idmap[p] = map[p][0]
+            if changes[p][0] == 'M' and not delta.has_key(p):
+                delta[p] = map[p][0]
+        
+        for x in sorted(delta.keys(), reverse=True):
+            if delta[x] is None:
+                del map[x]
+                for p in map.keys():
+                    if p.startswith(u"%s/" % x):
+                        del map[p]
 
-        map.update(dict([(x, (str(idmap[x]), revid)) for x in idmap]))
+        for x in sorted(delta.keys()):
+            if (delta[x] is not None and 
+                # special case - we change metadata in svn at the branch root path
+                # but that's not reflected as a bzr metadata change in bzr
+                (x != "" or not "" in map or map[x][1] == NULL_REVISION)):
+                map[x] = (str(delta[x]), revid)
 
-        # Mark all parent paths as changed
-        for p in idmap:
-            parts = p.split("/")
-            for j in range(1, len(parts)+1):
-                parent = "/".join(parts[0:len(parts)-j])
-                assert map.has_key(parent), "Parent item %s of %s doesn't exist in map" % (parent, p)
-                if map[parent][1] == revid:
-                    break
-                map[parent] = map[parent][0], revid
+
+class FileIdMapCache(object):
+
+    def __init__(self, cache_transport):
+        mapper = ConstantMapper("fileidmap-v%d" % FILEIDMAP_VERSION)
+        self.idmap_knit = make_file_factory(True, mapper)(cache_transport)
+
+    def save(self, revid, parent_revids, _map):
+        mutter('saving file id map for %r', revid)
+
+        for path, (id, created_revid)  in _map.items():
+            assert isinstance(path, unicode)
+            assert isinstance(id, str)
+            assert isinstance(created_revid, str)
+
+        self.idmap_knit.add_lines((revid,), [(r, ) for r in parent_revids], 
+                ["%s\t%s\t%s\n" % (urllib.quote(filename.encode("utf-8")), urllib.quote(_map[filename][0]), 
+                                        urllib.quote(_map[filename][1])) for filename in sorted(_map.keys())])
+
+    def load(self, revid):
+        map = {}
+        for ((create_revid,), line) in self.idmap_knit.annotate((revid,)):
+            (filename, id, create_revid) = line.rstrip("\n").split("\t", 3)
+            map[urllib.unquote(filename).decode("utf-8")] = (urllib.unquote(id), urllib.unquote(create_revid))
+            assert isinstance(map[urllib.unquote(filename).decode("utf-8")][0], str)
+
+        return map
 
 
 class CachingFileIdMap(object):
     """A file id map that uses a cache."""
     def __init__(self, cache_transport, actual):
-        self.idmap_knit = make_file_knit("fileidmap-v%d" % FILEIDMAP_VERSION, cache_transport, 0644, create=True)
+        self.cache = FileIdMapCache(cache_transport)
         self.actual = actual
         self.apply_changes = actual.apply_changes
+        self._use_text_revids = actual._use_text_revids
         self.repos = actual.repos
 
-    def save(self, revid, parent_revids, _map):
-        mutter('saving file id map for %r' % revid)
-                
-        self.idmap_knit.add_lines_with_ghosts(revid, parent_revids, 
-                ["%s\t%s\t%s\n" % (urllib.quote(filename), urllib.quote(_map[filename][0]), 
-                                        urllib.quote(_map[filename][1])) for filename in sorted(_map.keys())])
-
-    def load(self, revid):
-        map = {}
-        for line in self.idmap_knit.get_lines(revid):
-            (filename, id, create_revid) = line.rstrip("\n").split("\t", 3)
-            map[urllib.unquote(filename)] = (urllib.unquote(id), urllib.unquote(create_revid))
-            assert isinstance(map[urllib.unquote(filename)][0], str)
-
-        return map
-
     def get_map(self, uuid, revnum, branch, mapping):
         """Make sure the map is up to date until revnum."""
         # First, find the last cached map
@@ -224,11 +256,11 @@ class CachingFileIdMap(object):
         # No history -> empty map
         try:
             pb = ui.ui_factory.nested_progress_bar()
-            for revmeta in self.repos.iter_reverse_branch_changes(branch, revnum, mapping):
+            for revmeta in self.repos.iter_reverse_branch_changes(branch, revnum, to_revnum=0, mapping=mapping):
                 pb.update("fetching changes for file ids", revnum-revmeta.revnum, revnum)
                 revid = revmeta.get_revision_id(mapping)
                 try:
-                    map = self.load(revid)
+                    map = self.cache.load(revid)
                     # found the nearest cached map
                     next_parent_revs = [revid]
                     break
@@ -250,33 +282,23 @@ class CachingFileIdMap(object):
         pb = ui.ui_factory.nested_progress_bar()
 
         try:
-            i = 1
-            for revmeta in reversed(todo):
+            for i, revmeta in enumerate(reversed(todo)):
+                pb.update('generating file id map', i, len(todo))
                 revid = revmeta.get_revision_id(mapping)
-                expensive = False
                 def log_find_children(path, revnum):
-                    expensive = True
                     return self.repos._log.find_children(path, revnum)
 
-                (idmap, changes) = self.actual.apply_changes(self.repos.uuid, 
-                        revmeta.revnum, revmeta.branch_path, revmeta.paths, 
-                        mapping.import_fileid_map(revmeta.revprops, revmeta.fileprops), 
-                        mapping, log_find_children)
-                pb.update('generating file id map', i, len(todo))
-
-                parent_revs = next_parent_revs
+                (idmap, changes) = self.actual.apply_changes(
+                        revmeta, mapping, log_find_children)
 
                 self.actual.update_map(map, revid, idmap, changes)
+                self._use_text_revids(mapping, revmeta, map)
+
+                parent_revs = next_parent_revs
                        
-                saved = False
-                if i % 500 == 0 or expensive:
-                    self.save(revid, parent_revs, map)
-                    saved = True
+                self.cache.save(revid, parent_revs, map)
                 next_parent_revs = [revid]
-                i += 1
         finally:
             pb.finished()
-        if not saved:
-            self.save(revid, parent_revs, map)
         return map
 
