@@ -19,10 +19,8 @@ from bzrlib import osutils, registry
 from bzrlib.errors import InvalidRevisionId
 from bzrlib.trace import mutter
 
-from bzrlib.plugins.svn import version_info
+from bzrlib.plugins.svn import errors, properties, version_info
 import calendar
-import errors
-import svn
 import time
 import urllib
 
@@ -34,6 +32,8 @@ SVN_PROP_BZR_FILEIDS = 'bzr:file-ids'
 SVN_PROP_BZR_MERGE = 'bzr:merge'
 SVN_PROP_BZR_REVISION_INFO = 'bzr:revision-info'
 SVN_PROP_BZR_REVISION_ID = 'bzr:revision-id:v%d-' % MAPPING_VERSION
+SVN_PROP_BZR_TEXT_PARENTS = 'bzr:text-parents'
+SVN_PROP_BZR_LOG = 'bzr:log'
 
 SVN_REVPROP_BZR_COMMITTER = 'bzr:committer'
 SVN_REVPROP_BZR_FILEIDS = 'bzr:file-ids'
@@ -46,6 +46,7 @@ SVN_REVPROP_BZR_ROOT = 'bzr:root'
 SVN_REVPROP_BZR_SIGNATURE = 'bzr:gpg-signature'
 SVN_REVPROP_BZR_TIMESTAMP = 'bzr:timestamp'
 SVN_REVPROP_BZR_LOG = 'bzr:log'
+SVN_REVPROP_BZR_TEXT_PARENTS = 'bzr:text-parents'
 
 
 def escape_svn_path(x):
@@ -132,18 +133,23 @@ def parse_merge_property(line):
     :return: List of revisions merged
     """
     if ' ' in line:
-        mutter('invalid revision id %r in merged property, skipping' % line)
+        mutter('invalid revision id %r in merged property, skipping', line)
         return ()
 
     return tuple(filter(lambda x: x != "", line.split("\t")))
 
+
+def parse_svn_dateprop(date):
+    return (properties.time_from_cstring(date) / 1000000.0, 0)
+
+
 def parse_svn_revprops(svn_revprops, rev):
-    if svn_revprops.has_key(svn.core.SVN_PROP_REVISION_AUTHOR):
-        rev.committer = svn_revprops[svn.core.SVN_PROP_REVISION_AUTHOR]
+    if svn_revprops.has_key(properties.PROP_REVISION_AUTHOR):
+        rev.committer = svn_revprops[properties.PROP_REVISION_AUTHOR]
     else:
         rev.committer = ""
     
-    rev.message = svn_revprops.get(svn.core.SVN_PROP_REVISION_LOG)
+    rev.message = svn_revprops.get(properties.PROP_REVISION_LOG)
 
     if rev.message:
         try:
@@ -151,11 +157,8 @@ def parse_svn_revprops(svn_revprops, rev):
         except UnicodeDecodeError:
             pass
 
-    if svn_revprops.has_key(svn.core.SVN_PROP_REVISION_DATE):
-        rev.timestamp = 1.0 * svn.core.secs_from_timestr(svn_revprops[svn.core.SVN_PROP_REVISION_DATE], None)
-    else:
-        rev.timestamp = 0.0 # FIXME: Obtain repository creation time
-    rev.timezone = None
+    assert svn_revprops.has_key(properties.PROP_REVISION_DATE)
+    (rev.timestamp, rev.timezone) = parse_svn_dateprop(svn_revprops[properties.PROP_REVISION_DATE])
     rev.properties = {}
 
 
@@ -219,11 +222,12 @@ def generate_revision_metadata(timestamp, timezone, committer, revprops):
     if timestamp is not None:
         text += "timestamp: %s\n" % format_highres_date(timestamp, timezone) 
     if committer is not None:
-        text += "committer: %s\n" % committer
+        text += "committer: %s\n" % committer.encode("utf-8")
     if revprops is not None and revprops != {}:
         text += "properties: \n"
         for k, v in sorted(revprops.items()):
-            text += "\t%s: %s\n" % (k, v)
+            text += "\t%s: %s\n" % (k.encode("utf-8"), v.encode("utf-8"))
+    assert isinstance(text, str)
     return text
 
 
@@ -255,7 +259,7 @@ class BzrSvnMapping(object):
     def __init__(self):
         if (version_info[3] == 'exp' or self.experimental) and not BzrSvnMapping._warned_experimental:
             from bzrlib.trace import warning
-            warning("using experimental bzr-svn mappings; output may change between revisions")
+            warning("using experimental bzr-svn mappings; may break existing branches in the most horrible ways")
             BzrSvnMapping._warned_experimental = True
 
     @classmethod
@@ -324,13 +328,14 @@ class BzrSvnMapping(object):
         """
         raise NotImplementedError(self.generate_file_id)
 
-    def import_revision(self, revprops, fileprops, rev):
+    def import_revision(self, revprops, fileprops, uuid, branch, revnum, rev):
         """Update a Revision object from Subversion revision and branch 
         properties.
 
         :param revprops: Dictionary with Subversion revision properties.
         :param fileprops: Dictionary with Subversion file properties on the 
                           branch root.
+        :param revnum: Revision number in Subversion.
         :param rev: Revision object to import data into.
         """
         raise NotImplementedError(self.import_revision)
@@ -353,7 +358,7 @@ class BzrSvnMapping(object):
         """
         raise NotImplementedError(self.import_fileid_map)
 
-    def export_fileid_map(self, fileids, revprops, fileprops):
+    def export_fileid_map(self, can_use_custom_revprops, fileids, revprops, fileprops):
         """Adjust the properties for a file id map.
 
         :param fileids: Dictionary
@@ -362,14 +367,30 @@ class BzrSvnMapping(object):
         """
         raise NotImplementedError(self.export_fileid_map)
 
-    def export_revision(self, branch_root, timestamp, timezone, committer, revprops, 
-                        revision_id, revno, merges, fileprops):
+    def import_text_parents(self, revprops, fileprops):
+        """Obtain a text parent map from properties.
+
+        :param revprops: Subversion revision properties.
+        :param fileprops: File properties.
+        """
+        raise NotImplementedError(self.import_text_parents)
+
+    def export_text_parents(self, can_use_custom_revprops, text_parents, revprops, fileprops):
+        """Store a text parent map.
+
+        :param text_parents: Text parent map
+        :param revprops: Revision properties
+        :param fileprops: File properties
+        """
+        raise NotImplementedError(self.export_text_parents)
+
+    def export_revision(self, can_use_custom_revprops, branch_root, timestamp, timezone, committer, revprops, revision_id, revno, merges, fileprops):
         """Determines the revision properties and branch root file 
         properties.
         """
         raise NotImplementedError(self.export_revision)
 
-    def export_message(self, log, revprops, fileprops):
+    def export_message(self, can_use_custom_revprops, log, revprops, fileprops):
         raise NotImplementedError(self.export_message)
 
     def get_revision_id(self, branch_path, revprops, fileprops):
@@ -433,14 +454,26 @@ class BzrSvnMappingv2(BzrSvnMapping):
 def parse_fileid_property(text):
     ret = {}
     for line in text.splitlines():
-        (path, key) = line.split("\t", 2)
+        (path, key) = line.split("\t", 1)
         ret[urllib.unquote(path)] = osutils.safe_file_id(key)
     return ret
 
 
 def generate_fileid_property(fileids):
     """Marshall a dictionary with file ids."""
-    return "".join(["%s\t%s\n" % (urllib.quote(path), fileids[path]) for path in sorted(fileids.keys())])
+    return "".join(["%s\t%s\n" % (urllib.quote(path.encode("utf-8")), fileids[path]) for path in sorted(fileids.keys())])
+
+
+def parse_text_parents_property(text):
+    ret = {}
+    for line in text.splitlines():
+        (entry, parent_revid) = line.split("\t", 1)
+        ret[urllib.unquote(entry)] = osutils.safe_revision_id(parent_revid)
+    return ret
+
+
+def generate_text_parents_property(text_parents):
+    return "".join(["%s\t%s\n" % (urllib.quote(path.encode("utf-8")), text_parents[path]) for path in sorted(text_parents.keys())])
 
 
 class BzrSvnMappingFileProps(object):
@@ -449,12 +482,26 @@ class BzrSvnMappingFileProps(object):
         """Whether this mapping can be used with custom file properties."""
         return True
 
-    def import_revision(self, svn_revprops, fileprops, rev):
+    def import_revision(self, svn_revprops, fileprops, uuid, branch, revnum, rev):
         parse_svn_revprops(svn_revprops, rev)
+        if SVN_PROP_BZR_LOG in fileprops:
+            rev.message = fileprops[SVN_PROP_BZR_LOG]
         metadata = fileprops.get(SVN_PROP_BZR_REVISION_INFO)
         if metadata is not None:
             parse_revision_metadata(metadata, rev)
 
+    def import_text_parents(self, svn_revprops, fileprops):
+        metadata = fileprops.get(SVN_PROP_BZR_TEXT_PARENTS)
+        if metadata is None:
+            return {}
+        return parse_text_parents_property(metadata)
+
+    def export_text_parents(self, can_use_custom_revprops, text_parents, svn_revprops, fileprops):
+        if text_parents != {}:
+            fileprops[SVN_PROP_BZR_TEXT_PARENTS] = generate_text_parents_property(text_parents)
+        else:
+            fileprops[SVN_PROP_BZR_TEXT_PARENTS] = ""
+
     def get_rhs_parents(self, branch_path, revprops, fileprops):
         bzr_merges = fileprops.get(SVN_PROP_BZR_ANCESTRY+str(self.scheme), None)
         if bzr_merges is not None:
@@ -485,7 +532,7 @@ class BzrSvnMappingFileProps(object):
 
         return svnprops
  
-    def export_revision(self, branch_root, timestamp, timezone, committer, revprops, revision_id, revno, merges, old_fileprops):
+    def export_revision(self, can_use_custom_revprops, branch_root, timestamp, timezone, committer, revprops, revision_id, revno, merges, old_fileprops):
 
         # Keep track of what Subversion properties to set later on
         fileprops = {}
@@ -503,6 +550,9 @@ class BzrSvnMappingFileProps(object):
 
         return ({}, fileprops)
 
+    def export_message(self, can_use_custom_revprops, message, revprops, fileprops):
+        fileprops[SVN_PROP_BZR_LOG] = message.encode("utf-8")
+
     def is_bzr_revision(self, revprops, fileprops):
         return fileprops.has_key(SVN_PROP_BZR_REVISION_ID+str(self.scheme))
 
@@ -522,20 +572,21 @@ class BzrSvnMappingFileProps(object):
             mutter(str(e))
             return (None, None)
 
-    def export_fileid_map(self, fileids, revprops, fileprops):
+    def export_fileid_map(self, can_use_custom_revprops, fileids, revprops, fileprops):
         if fileids != {}:
             file_id_text = generate_fileid_property(fileids)
             fileprops[SVN_PROP_BZR_FILEIDS] = file_id_text
         else:
             fileprops[SVN_PROP_BZR_FILEIDS] = ""
 
+
 class BzrSvnMappingRevProps(object):
     @classmethod
     def supports_custom_revprops(cls):
         """Whether this mapping can be used with custom revision properties."""
         return True
 
-    def import_revision(self, svn_revprops, fileprops, rev):
+    def import_revision(self, svn_revprops, fileprops, uuid, branch, revnum, rev):
         parse_svn_revprops(svn_revprops, rev)
         parse_bzr_svn_revprops(svn_revprops, rev)
 
@@ -544,6 +595,14 @@ class BzrSvnMappingRevProps(object):
             return {}
         return parse_fileid_property(svn_revprops[SVN_REVPROP_BZR_FILEIDS])
 
+    def import_text_parents(self, svn_revprops, fileprops):
+        if not svn_revprops.has_key(SVN_REVPROP_BZR_TEXT_PARENTS):
+            return {}
+        return parse_text_parents_property(svn_revprops[SVN_REVPROP_BZR_TEXT_PARENTS])
+
+    def export_text_parents(self, can_use_custom_revprops, text_parents, svn_revprops, fileprops):
+        svn_revprops[SVN_REVPROP_BZR_TEXT_PARENTS] = generate_text_parents_property(text_parents)
+
     def get_rhs_parents(self, branch_path, svn_revprops, 
                         fileprops):
         if svn_revprops[SVN_REVPROP_BZR_ROOT] != branch:
@@ -562,13 +621,11 @@ class BzrSvnMappingRevProps(object):
             return (revno, revid)
         return (None, None)
 
-    def export_message(self, message, revprops, fileprops):
+    def export_message(self, can_use_custom_revprops, message, revprops, fileprops):
         revprops[SVN_REVPROP_BZR_LOG] = message.encode("utf-8")
 
-    def export_revision(self, branch_root, timestamp, timezone, committer, 
-                        revprops, revision_id, revno, merges, 
-                        fileprops):
-        svn_revprops = {SVN_REVPROP_BZR_MAPPING_VERSION: str(MAPPING_VERSION)}
+    def export_revision(self, can_use_custom_revprops, branch_root, timestamp, timezone, committer, revprops, revision_id, revno, merges, fileprops):
+        svn_revprops = {}
 
         if timestamp is not None:
             svn_revprops[SVN_REVPROP_BZR_TIMESTAMP] = format_highres_date(timestamp, timezone)
@@ -578,7 +635,7 @@ class BzrSvnMappingRevProps(object):
 
         if revprops is not None:
             for name, value in revprops.items():
-                svn_revprops[SVN_REVPROP_BZR_REVPROP_PREFIX+name] = value
+                svn_revprops[SVN_REVPROP_BZR_REVPROP_PREFIX+name] = value.encode("utf-8")
 
         svn_revprops[SVN_REVPROP_BZR_ROOT] = branch_root
 
@@ -591,7 +648,7 @@ class BzrSvnMappingRevProps(object):
 
         return (svn_revprops, {})
 
-    def export_fileid_map(self, fileids, revprops, fileprops):
+    def export_fileid_map(self, can_use_custom_revprops, fileids, revprops, fileprops):
         revprops[SVN_REVPROP_BZR_FILEIDS] = generate_fileid_property(fileids)
 
     def get_rhs_ancestors(self, branch_path, revprops, fileprops):
@@ -606,6 +663,14 @@ class BzrSvnMappingv4(BzrSvnMappingRevProps):
     def supports_roundtripping():
         return True
 
+    def import_revision(self, svn_revprops, fileprops, uuid, branch, revnum, rev):
+        super(BzrSvnMappingv4, self).import_revision(svn_revprops, fileprops, uuid, branch, revnum, rev)
+
+    def export_revision(self, can_use_custom_revprops, branch_root, timestamp, timezone, committer, revprops, revision_id, revno, merges, fileprops):
+        (revprops, fileprops) = BzrSvnMappingRevProps.export_revision(self, can_use_custom_revprops, branch_root, timestamp, timezone, committer, revprops, revision_id, revno, merges, fileprops)
+        revprops[SVN_REVPROP_BZR_MAPPING_VERSION] = "4"
+        return (revprops, fileprops)
+
     @classmethod
     def parse_revision_id(cls, revid):
         assert isinstance(revid, str)
@@ -683,7 +748,7 @@ def parse_revision_id(revid):
     """Try to parse a Subversion revision id.
     
     :param revid: Revision id to parse
-    :return: tuple with (uuid, branch_path, mapping)
+    :return: tuple with (uuid, branch_path, revno, mapping)
     """
     if not revid.startswith("svn-"):
         raise InvalidRevisionId(revid, None)
@@ -693,3 +758,5 @@ def parse_revision_id(revid):
 
 def get_default_mapping():
     return mapping_registry.get_default()
+
+
