@@ -15,24 +15,21 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 """Branching scheme implementations."""
 
-from bzrlib import ui
+from bzrlib import ui, urlutils
 from bzrlib.errors import BzrError
 from bzrlib.trace import mutter
 
-from bzrlib.errors import NotBranchError
-
 from base64 import urlsafe_b64decode, urlsafe_b64encode
-from errors import InvalidSvnBranchPath
-import util
+from bzrlib.plugins.svn.errors import InvalidSvnBranchPath
+from bzrlib.plugins.svn import properties
 import bz2
-import urllib
 
 class BranchingScheme(object):
     """ Divides SVN repository data up into branches. Since there
     is no proper way to do this, there are several subclasses of this class
     each of which handles a particular convention that may be in use.
     """
-    def is_branch(self, path):
+    def is_branch(self, path, project=None):
         """Check whether a location refers to a branch.
         
         :param path: Path to check.
@@ -43,7 +40,21 @@ class BranchingScheme(object):
         """Split up a Subversion path into a branch-path and inside-branch path.
 
         :param path: Path to split up.
-        :return: Tuple with branch-path and inside-branch path.
+        :return: Tuple with project name,branch-path and inside-branch path.
+        """
+        raise NotImplementedError
+
+    def get_tag_path(self, name):
+        """Find the path for a tag.
+
+        :param name: Tag name.
+        """
+        raise NotImplementedError
+
+    def get_branch_path(self, name):
+        """Find the path for a named branch.
+
+        :param name: Branch name.
         """
         raise NotImplementedError
 
@@ -77,7 +88,7 @@ class BranchingScheme(object):
 
         raise UnknownBranchingScheme(name)
 
-    def is_branch_parent(self, path):
+    def is_branch_parent(self, path, project=None):
         """Check whether the specified path is the parent directory of branches.
         The path may not be a branch itself.
         
@@ -86,7 +97,7 @@ class BranchingScheme(object):
         """
         raise NotImplementedError
 
-    def is_tag_parent(self, path):
+    def is_tag_parent(self, path, project=None):
         """Check whether the specified path is the parent directory of tags.
         The path may not be a tag itself.
         
@@ -95,7 +106,7 @@ class BranchingScheme(object):
         """
         raise NotImplementedError
 
-    def is_tag(self, path):
+    def is_tag(self, path, project=None):
         """Check whether the specified path is a tag 
         according to this branching scheme.
 
@@ -138,7 +149,7 @@ def prop_name_quote(text):
 class ListBranchingScheme(BranchingScheme):
     """Branching scheme that keeps a list of branch paths, including 
     wildcards."""
-    def __init__(self, branch_list):
+    def __init__(self, branch_list, tag_list=[]):
         """Create new ListBranchingScheme instance.
 
         :param branch_list: List of know branch locations.
@@ -147,12 +158,13 @@ class ListBranchingScheme(BranchingScheme):
         if isinstance(branch_list, str):
             branch_list = bz2.decompress(prop_name_unquote(branch_list.encode("ascii"))).splitlines()
         self.branch_list = [p.strip("/") for p in branch_list]
+        self.tag_list = tag_list
         self.split_branch_list = [p.split("/") for p in self.branch_list]
 
     def __str__(self):
         return "list-%s" % prop_name_quote(bz2.compress("".join(map(lambda x:x+"\n", self.branch_list))))
             
-    def is_tag(self, path):
+    def is_tag(self, path, project=None):
         """See BranchingScheme.is_tag()."""
         return False
 
@@ -165,7 +177,7 @@ class ListBranchingScheme(BranchingScheme):
                 return False
         return True
 
-    def is_branch(self, path):
+    def is_branch(self, path, project=None):
         """See BranchingScheme.is_branch()."""
         parts = path.strip("/").split("/")
         for pattern in self.split_branch_list:
@@ -180,20 +192,22 @@ class ListBranchingScheme(BranchingScheme):
         for pattern in self.split_branch_list:
             if self._pattern_cmp(parts[:len(pattern)], pattern):
                 return ("/".join(parts[:len(pattern)]), 
+                        "/".join(parts[:len(pattern)]), 
                         "/".join(parts[len(pattern):]))
         raise InvalidSvnBranchPath(path, self)
 
     def __eq__(self, other):
-        return self.branch_list == other.branch_list
+        return (self.branch_list == other.branch_list and \
+                self.tag_list == other.tag_list)
 
     def to_lines(self):
         return self.branch_list
 
-    def is_tag_parent(self, path):
+    def is_tag_parent(self, path, project=None):
         # ListBranchingScheme doesn't have tags
         return False
 
-    def is_branch_parent(self, path):
+    def is_branch_parent(self, path, project=None):
         parts = path.strip("/").split("/")
         for pattern in self.split_branch_list:
             if len(parts) == len(pattern):
@@ -209,17 +223,19 @@ class NoBranchingScheme(ListBranchingScheme):
     def __init__(self):
         ListBranchingScheme.__init__(self, [""])
 
-    def is_branch(self, path):
+    def is_branch(self, path, project=None):
         """See BranchingScheme.is_branch()."""
-        return path.strip("/") == ""
+        if project is None or project == "":
+            return path.strip("/") == ""
+        return False
 
-    def is_tag(self, path):
+    def is_tag(self, path, project=None):
         return False
 
     def unprefix(self, path):
         """See BranchingScheme.unprefix()."""
         assert isinstance(path, str)
-        return ("", path.strip("/"))
+        return ("", "", path.strip("/"))
 
     def __str__(self):
         return "none"
@@ -227,10 +243,10 @@ class NoBranchingScheme(ListBranchingScheme):
     def __repr__(self):
         return "%s()" % self.__class__.__name__
 
-    def is_branch_parent(self, path):
+    def is_branch_parent(self, path, project=None):
         return False
 
-    def is_tag_parent(self, path):
+    def is_tag_parent(self, path, project=None):
         return False
 
 
@@ -243,12 +259,35 @@ class TrunkBranchingScheme(ListBranchingScheme):
         self.level = level
         ListBranchingScheme.__init__(self,
             ["*/" * level + "trunk",
-             "*/" * level + "branches/*",
-             "*/" * level + "tags/*"])
+             "*/" * level + "branches/*"])
+        self.tag_list = ["*/" * level + "tags/*"]
 
-    def is_branch(self, path):
+    def get_tag_path(self, name, project=""):
+        assert isinstance(name, unicode)
+        if project == "":
+            return urlutils.join("tags", name.encode("utf-8"))
+        return urlutils.join(project, "tags", name.encode("utf-8"))
+
+    def get_branch_path(self, name, project=""):
+        # Only implemented for level 0
+        if name == "trunk":
+            if project == "":
+                return "trunk"
+            else:
+                return urlutils.join(project, "trunk")
+        else:
+            if project == "":
+                return urlutils.join("branches", name)
+            else:
+                return urlutils.join(project, "branches", name)
+
+    def is_branch(self, path, project=None):
         """See BranchingScheme.is_branch()."""
         parts = path.strip("/").split("/")
+
+        if project is not None and project != "/".join(parts[0:self.level]):
+            return False
+
         if len(parts) == self.level+1 and parts[self.level] == "trunk":
             return True
 
@@ -257,9 +296,13 @@ class TrunkBranchingScheme(ListBranchingScheme):
 
         return False
 
-    def is_tag(self, path):
+    def is_tag(self, path, project=None):
         """See BranchingScheme.is_tag()."""
         parts = path.strip("/").split("/")
+
+        if project is not None and project != "/".join(parts[0:self.level]):
+            return False
+
         if len(parts) == self.level+2 and \
            (parts[self.level] == "tags"):
             return True
@@ -274,11 +317,13 @@ class TrunkBranchingScheme(ListBranchingScheme):
             raise InvalidSvnBranchPath(path, self)
 
         if parts[self.level] == "trunk" or parts[self.level] == "hooks":
-            return ("/".join(parts[0:self.level+1]).strip("/"), 
+            return ("/".join(parts[0:self.level]).strip("/"),
+                    "/".join(parts[0:self.level+1]).strip("/"), 
                     "/".join(parts[self.level+1:]).strip("/"))
         elif ((parts[self.level] == "tags" or parts[self.level] == "branches") and 
               len(parts) >= self.level+2):
-            return ("/".join(parts[0:self.level+2]).strip("/"), 
+            return ("/".join(parts[0:self.level]).strip("/"),
+                    "/".join(parts[0:self.level+2]).strip("/"), 
                     "/".join(parts[self.level+2:]).strip("/"))
         else:
             raise InvalidSvnBranchPath(path, self)
@@ -289,15 +334,15 @@ class TrunkBranchingScheme(ListBranchingScheme):
     def __repr__(self):
         return "%s(%d)" % (self.__class__.__name__, self.level)
 
-    def is_branch_parent(self, path):
+    def is_branch_parent(self, path, project=None):
         parts = path.strip("/").split("/")
         if len(parts) <= self.level:
             return True
-        return self.is_branch(path+"/trunk")
+        return self.is_branch(path+"/trunk", project)
 
-    def is_tag_parent(self, path):
+    def is_tag_parent(self, path, project=None):
         parts = path.strip("/").split("/")
-        return self.is_tag(path+"/aname")
+        return self.is_tag(path+"/aname", project)
 
 
 
@@ -319,11 +364,11 @@ class SingleBranchingScheme(ListBranchingScheme):
             raise BzrError("NoBranchingScheme should be used")
         ListBranchingScheme.__init__(self, [self.path])
 
-    def is_branch(self, path):
+    def is_branch(self, path, project=None):
         """See BranchingScheme.is_branch()."""
         return self.path == path.strip("/")
 
-    def is_tag(self, path):
+    def is_tag(self, path, project=None):
         """See BranchingScheme.is_tag()."""
         return False
 
@@ -334,11 +379,12 @@ class SingleBranchingScheme(ListBranchingScheme):
         if not path.startswith(self.path):
             raise InvalidSvnBranchPath(path, self)
 
-        return (path[0:len(self.path)].strip("/"), 
+        return (self.path, 
+                path[0:len(self.path)].strip("/"), 
                 path[len(self.path):].strip("/"))
 
     def __str__(self):
-        if util.is_valid_property_name(self.path):
+        if properties.is_valid_property_name(self.path):
             return "single-%s" % self.path
         else:
             return "single1-%s" % prop_name_quote(self.path)
@@ -346,12 +392,12 @@ class SingleBranchingScheme(ListBranchingScheme):
     def __repr__(self):
         return "%s(%r)" % (self.__class__.__name__, self.path)
 
-    def is_branch_parent(self, path):
+    def is_branch_parent(self, path, project=None):
         if not "/" in self.path:
             return False
-        return self.is_branch(path+"/"+self.path.split("/")[-1])
+        return self.is_branch(path+"/"+self.path.split("/")[-1], project=None)
 
-    def is_tag_parent(self, path):
+    def is_tag_parent(self, path, project=None):
         return False
 
 
@@ -431,7 +477,9 @@ def guess_scheme_from_history(changed_paths, last_revnum,
     :param last_revnum: Number of entries in changed_paths.
     :param relpath: Branch path that should be accepted by the branching 
                     scheme as a branch.
-    :return: Branching scheme instance that matches best.
+    :return: Tuple with branching scheme that best matches history and 
+             branching scheme instance that best matches but also considers
+             relpath a valid branch path.
     """
     potentials = {}
     pb = ui.ui_factory.nested_progress_bar()
@@ -441,6 +489,8 @@ def guess_scheme_from_history(changed_paths, last_revnum,
             assert isinstance(revpaths, dict)
             pb.update("analyzing repository layout", last_revnum-revnum, 
                       last_revnum)
+            if revpaths == {}:
+                continue
             for path in find_commit_paths([revpaths]):
                 scheme = guess_scheme_from_path(path)
                 if not potentials.has_key(str(scheme)):
@@ -455,17 +505,22 @@ def guess_scheme_from_history(changed_paths, last_revnum,
 
     mutter('potential branching schemes: %r' % entries)
 
+    if len(entries) > 0:
+        best_match = scheme_cache[entries[0][0]]
+    else:
+        best_match = None
+
     if relpath is None:
-        if len(entries) == 0:
-            return NoBranchingScheme()
-        return scheme_cache[entries[0][0]]
+        if best_match is None:
+            return (None, NoBranchingScheme())
+        return (best_match, best_match)
 
     for (schemename, _) in entries:
         scheme = scheme_cache[schemename]
         if scheme.is_branch(relpath):
-            return scheme
+            return (best_match, scheme)
 
-    return guess_scheme_from_branch_path(relpath)
+    return (best_match, guess_scheme_from_branch_path(relpath))
 
 
 def scheme_from_branch_list(branch_list):
