@@ -16,9 +16,10 @@
 
 """Branch tests."""
 
+from bzrlib import urlutils
 from bzrlib.branch import Branch
 from bzrlib.bzrdir import BzrDir
-from bzrlib.errors import NoSuchFile, NoSuchRevision, NotBranchError
+from bzrlib.errors import NoSuchFile, NoSuchRevision, NotBranchError, NoSuchTag
 from bzrlib.repository import Repository
 from bzrlib.revision import NULL_REVISION
 from bzrlib.trace import mutter
@@ -26,39 +27,136 @@ from bzrlib.trace import mutter
 import os
 from unittest import TestCase
 
-from branch import FakeControlFiles, SvnBranchFormat
-from convert import load_dumpfile
-from mapping import SVN_PROP_BZR_REVISION_ID
-from mapping3 import BzrSvnMappingv3FileProps
-from mapping3.scheme import TrunkBranchingScheme
-from tests import TestCaseWithSubversionRepository
+from bzrlib.plugins.svn import core
+from bzrlib.plugins.svn.branch import FakeControlFiles, SvnBranchFormat
+from bzrlib.plugins.svn.convert import load_dumpfile
+from bzrlib.plugins.svn.mapping import SVN_PROP_BZR_REVISION_ID
+from bzrlib.plugins.svn.mapping3 import BzrSvnMappingv3FileProps
+from bzrlib.plugins.svn.mapping3.scheme import TrunkBranchingScheme
+from bzrlib.plugins.svn.tests import TestCaseWithSubversionRepository
 
 class WorkingSubversionBranch(TestCaseWithSubversionRepository):
     def test_last_rev_rev_hist(self):
-        repos_url = self.make_client("a", "dc")
+        repos_url = self.make_repository("a")
         branch = Branch.open(repos_url)
         branch.revision_history()
         self.assertEqual(branch.generate_revision_id(0), branch.last_revision())
 
     def test_get_branch_path_root(self):
-        repos_url = self.make_client("a", "dc")
+        repos_url = self.make_repository("a")
         branch = Branch.open(repos_url)
         self.assertEqual("", branch.get_branch_path())
 
+    def test_tags_dict(self):
+        repos_url = self.make_repository("a")
+       
+        dc = self.get_commit_editor(repos_url)
+        tags = dc.add_dir("tags")
+        tags.add_dir("tags/foo")
+        dc.add_dir("trunk")
+        dc.close()
+
+        b = Branch.open(repos_url + "/trunk")
+        self.assertEquals(["foo"], b.tags.get_tag_dict().keys())
+
+    def test_tag_set(self):
+        repos_url = self.make_repository('a')
+
+        dc = self.get_commit_editor(repos_url)
+        dc.add_dir("trunk")
+        dc.add_dir("tags")
+        dc.close()
+
+        dc = self.get_commit_editor(repos_url)
+        trunk = dc.open_dir("trunk")
+        trunk.add_file("trunk/bla").modify()
+        dc.close()
+
+        b = Branch.open(repos_url + "/trunk")
+        b.tags.set_tag(u"mytag", b.repository.generate_revision_id(1, "trunk", b.repository.get_mapping()))
+
+        self.assertEquals(core.NODE_DIR, 
+                b.repository.transport.check_path("tags/mytag", 3))
+
+    def test_tags_delete(self):
+        repos_url = self.make_repository("a")
+       
+        dc = self.get_commit_editor(repos_url)
+        tags = dc.add_dir("tags")
+        tags.add_dir("tags/foo")
+        dc.add_dir("trunk")
+        dc.close()
+
+        b = Branch.open(repos_url + "/trunk")
+        self.assertEquals(["foo"], b.tags.get_tag_dict().keys())
+        b.tags.delete_tag(u"foo")
+        b = Branch.open(repos_url + "/trunk")
+        self.assertEquals([], b.tags.get_tag_dict().keys())
+
+    def test_tag_lookup(self):
+        repos_url = self.make_repository("a")
+       
+        dc = self.get_commit_editor(repos_url)
+        tags = dc.add_dir("tags")
+        tags.add_dir("tags/foo")
+        dc.add_dir("trunk")
+        dc.close()
+
+        b = Branch.open(repos_url + "/trunk")
+        self.assertEquals("", b.project)
+        self.assertEquals(b.repository.generate_revision_id(1, "tags/foo", b.repository.get_mapping()), b.tags.lookup_tag("foo"))
+
+    def test_tag_lookup_nonexistant(self):
+        repos_url = self.make_repository("a")
+
+        dc = self.get_commit_editor(repos_url)
+        dc.add_dir("trunk")
+        dc.close()
+       
+        b = Branch.open(repos_url + "/trunk")
+        self.assertRaises(NoSuchTag, b.tags.lookup_tag, "foo")
+
+    def test_tags_delete_nonexistent(self):
+        repos_url = self.make_repository("a")
+
+        dc = self.get_commit_editor(repos_url)
+        dc.add_dir("trunk")
+        dc.close()
+       
+        b = Branch.open(repos_url + "/trunk")
+        self.assertRaises(NoSuchTag, b.tags.delete_tag, u"foo")
+
+    def test_get_branch_path_old(self):
+        repos_url = self.make_repository("a")
+
+        dc = self.get_commit_editor(repos_url)
+        dc.add_dir("trunk")
+        dc.close()
+
+        dc = self.get_commit_editor(repos_url)
+        dc.add_dir("trunk2", "trunk", 1)
+        dc.close()
+
+        branch = Branch.open(urlutils.join(repos_url, "trunk2"))
+        self.assertEqual("trunk2", branch.get_branch_path(2))
+        self.assertEqual("trunk", branch.get_branch_path(1))
+
     def test_get_branch_path_subdir(self):
-        repos_url = self.make_client("a", "dc")
-        self.build_tree({"dc/trunk": None})
-        self.client_add("dc/trunk")
-        self.client_commit("dc", "Add branch")
+        repos_url = self.make_repository("a")
+
+        dc = self.get_commit_editor(repos_url)
+        dc.add_dir("trunk")
+        dc.close()
+
         branch = Branch.open(repos_url+"/trunk")
         self.assertEqual("trunk", branch.get_branch_path())
 
     def test_open_nonexistant(self):
-        repos_url = self.make_client("a", "dc")
+        repos_url = self.make_repository("a")
         self.assertRaises(NotBranchError, Branch.open, repos_url + "/trunk")
 
     def test_last_rev_rev_info(self):
-        repos_url = self.make_client("a", "dc")
+        repos_url = self.make_repository("a")
         branch = Branch.open(repos_url)
         self.assertEqual((1, branch.generate_revision_id(0)),
                 branch.last_revision_info())
@@ -67,35 +165,34 @@ class WorkingSubversionBranch(TestCaseWithSubversionRepository):
                 branch.last_revision_info())
 
     def test_lookup_revision_id_unknown(self):
-        repos_url = self.make_client("a", "dc")
+        repos_url = self.make_repository("a")
         branch = Branch.open(repos_url)
         self.assertRaises(NoSuchRevision, 
                 lambda: branch.lookup_revision_id("bla"))
 
     def test_lookup_revision_id(self):
-        repos_url = self.make_client("a", "dc")
+        repos_url = self.make_repository("a")
         branch = Branch.open(repos_url)
         self.assertEquals(0, 
                 branch.lookup_revision_id(branch.last_revision()))
 
     def test_set_parent(self):
-        repos_url = self.make_client('a', 'dc')
+        repos_url = self.make_repository('a')
         branch = Branch.open(repos_url)
         branch.set_parent("foobar")
 
     def test_num_revnums(self):
-        repos_url = self.make_client('a', 'dc')
-        bzrdir = BzrDir.open("svn+"+repos_url)
+        repos_url = self.make_repository('a')
+        bzrdir = BzrDir.open(repos_url)
         branch = bzrdir.open_branch()
         self.assertEqual(branch.generate_revision_id(0),
                          branch.last_revision())
 
-        self.build_tree({'dc/foo': "data"})
-        self.client_add("dc/foo")
-        self.client_commit("dc", "My Message")
-        self.client_update("dc")
+        dc = self.get_commit_editor(repos_url)
+        dc.add_file("foo").modify()
+        dc.close()
         
-        bzrdir = BzrDir.open("svn+"+repos_url)
+        bzrdir = BzrDir.open(repos_url)
         branch = bzrdir.open_branch()
         repos = bzrdir.find_repository()
         
@@ -104,71 +201,100 @@ class WorkingSubversionBranch(TestCaseWithSubversionRepository):
         self.assertEqual(repos.generate_revision_id(1, "", mapping), 
                 branch.last_revision())
 
-        self.build_tree({'dc/foo': "data2"})
-        self.client_commit("dc", "My Message")
+        dc = self.get_commit_editor(repos_url)
+        dc.open_file("foo").modify()
+        dc.close()
 
-        branch = Branch.open("svn+"+repos_url)
-        repos = Repository.open("svn+"+repos_url)
+        branch = Branch.open(repos_url)
+        repos = Repository.open(repos_url)
 
         self.assertEqual(repos.generate_revision_id(2, "", mapping),
                 branch.last_revision())
 
-    def test_set_revision_history(self):
-        repos_url = self.make_client('a', 'dc')
-        branch = Branch.open("svn+"+repos_url)
+    def test_set_revision_history_empty(self):
+        repos_url = self.make_repository('a')
+        branch = Branch.open(repos_url)
         self.assertRaises(NotImplementedError, branch.set_revision_history, [])
 
+    def test_set_revision_history_ghost(self):
+        repos_url = self.make_repository('a')
+
+        dc = self.get_commit_editor(repos_url)
+        trunk = dc.add_dir("trunk")
+        trunk.add_file('trunk/foo').modify()
+        dc.close()
+
+        branch = Branch.open(repos_url+"/trunk")
+        self.assertRaises(NotImplementedError, branch.set_revision_history, ["nonexistantt"])
+
+    def test_set_revision_history(self):
+        repos_url = self.make_repository('a')
+
+        dc = self.get_commit_editor(repos_url)
+        trunk = dc.add_dir("trunk")
+        trunk.add_file('trunk/foo').modify()
+        dc.close()
+
+        dc = self.get_commit_editor(repos_url)
+        trunk = dc.open_dir("trunk")
+        trunk.add_file('trunk/bla').modify()
+        dc.close()
+
+        dc = self.get_commit_editor(repos_url)
+        trunk = dc.open_dir("trunk")
+        trunk.add_file('trunk/bar').modify()
+        dc.close()
+
+        branch = Branch.open(repos_url+"/trunk")
+        orig_history = branch.revision_history()
+        branch.set_revision_history(orig_history[:-1])
+        self.assertEquals(orig_history[:-1], branch.revision_history())
+
     def test_break_lock(self):
-        repos_url = self.make_client('a', 'dc')
-        branch = Branch.open("svn+"+repos_url)
+        repos_url = self.make_repository('a')
+        branch = Branch.open(repos_url)
         branch.control_files.break_lock()
 
     def test_repr(self):
-        repos_url = self.make_client('a', 'dc')
-        branch = Branch.open("svn+"+repos_url)
-        self.assertEqual("SvnBranch('svn+%s')" % repos_url, branch.__repr__())
+        repos_url = self.make_repository('a')
+        branch = Branch.open(repos_url)
+        self.assertEqual("SvnBranch('%s')" % repos_url, branch.__repr__())
 
     def test_get_physical_lock_status(self):
-        repos_url = self.make_client('a', 'dc')
-        branch = Branch.open("svn+"+repos_url)
+        repos_url = self.make_repository('a')
+        branch = Branch.open(repos_url)
         self.assertFalse(branch.get_physical_lock_status())
 
     def test_set_push_location(self):
-        repos_url = self.make_client('a', 'dc')
-        branch = Branch.open("svn+"+repos_url)
+        repos_url = self.make_repository('a')
+        branch = Branch.open(repos_url)
         self.assertRaises(NotImplementedError, branch.set_push_location, [])
 
     def test_get_parent(self):
-        repos_url = self.make_client('a', 'dc')
-        branch = Branch.open("svn+"+repos_url)
+        repos_url = self.make_repository('a')
+        branch = Branch.open(repos_url)
         self.assertEqual(None, branch.get_parent())
 
-    def test_append_revision(self):
-        repos_url = self.make_client('a', 'dc')
-        branch = Branch.open("svn+"+repos_url)
-        branch.append_revision([])
-
     def test_get_push_location(self):
-        repos_url = self.make_client('a', 'dc')
-        branch = Branch.open("svn+"+repos_url)
+        repos_url = self.make_repository('a')
+        branch = Branch.open(repos_url)
         self.assertIs(None, branch.get_push_location())
 
     def test_revision_history(self):
-        repos_url = self.make_client('a', 'dc')
+        repos_url = self.make_repository('a')
 
-        branch = Branch.open("svn+"+repos_url)
+        branch = Branch.open(repos_url)
         self.assertEqual([branch.generate_revision_id(0)], 
                 branch.revision_history())
 
-        self.build_tree({'dc/foo': "data"})
-        self.client_add("dc/foo")
-        self.client_set_prop("dc", SVN_PROP_BZR_REVISION_ID+"none", 
+        dc = self.get_commit_editor(repos_url)
+        dc.add_file("foo").modify()
+        dc.change_prop(SVN_PROP_BZR_REVISION_ID+"none", 
                 "42 mycommit\n")
-        self.client_commit("dc", "My Message")
-        self.client_update("dc")
+        dc.close()
         
-        branch = Branch.open("svn+"+repos_url)
-        repos = Repository.open("svn+"+repos_url)
+        branch = Branch.open(repos_url)
+        repos = Repository.open(repos_url)
         
         mapping = repos.get_mapping()
 
@@ -176,11 +302,12 @@ class WorkingSubversionBranch(TestCaseWithSubversionRepository):
                     repos.generate_revision_id(1, "", mapping)], 
                 branch.revision_history())
 
-        self.build_tree({'dc/foo': "data34"})
-        self.client_commit("dc", "My Message")
+        dc = self.get_commit_editor(repos_url)
+        dc.open_file("foo").modify()
+        dc.close()
 
-        branch = Branch.open("svn+"+repos_url)
-        repos = Repository.open("svn+"+repos_url)
+        branch = Branch.open(repos_url)
+        repos = Repository.open(repos_url)
 
         mapping = repos.get_mapping()
 
@@ -192,74 +319,80 @@ class WorkingSubversionBranch(TestCaseWithSubversionRepository):
 
     def test_revision_id_to_revno_none(self):
         """The None revid should map to revno 0."""
-        repos_url = self.make_client('a', 'dc')
+        repos_url = self.make_repository('a')
         branch = Branch.open(repos_url)
         self.assertEquals(0, branch.revision_id_to_revno(NULL_REVISION))
 
     def test_revision_id_to_revno_nonexistant(self):
         """revision_id_to_revno() should raise NoSuchRevision if
         the specified revision did not exist in the branch history."""
-        repos_url = self.make_client('a', 'dc')
+        repos_url = self.make_repository('a')
         branch = Branch.open(repos_url)
         self.assertRaises(NoSuchRevision, branch.revision_id_to_revno, "bla")
     
     def test_revision_id_to_revno_simple(self):
-        repos_url = self.make_client('a', 'dc')
-        self.build_tree({'dc/foo': "data"})
-        self.client_add("dc/foo")
-        self.client_set_prop("dc", "bzr:revision-id:v3-none", 
+        repos_url = self.make_repository('a')
+
+        dc = self.get_commit_editor(repos_url)
+        dc.add_file("foo").modify()
+        dc.change_prop("bzr:revision-id:v3-none", 
                             "2 myrevid\n")
-        self.client_commit("dc", "My Message")
+        dc.close()
+
         branch = Branch.open(repos_url)
         self.assertEquals(2, branch.revision_id_to_revno("myrevid"))
 
     def test_revision_id_to_revno_older(self):
-        repos_url = self.make_client('a', 'dc')
-        self.build_tree({'dc/foo': "data"})
-        self.client_add("dc/foo")
-        self.client_set_prop("dc", "bzr:revision-id:v3-none", 
+        repos_url = self.make_repository('a')
+
+        dc = self.get_commit_editor(repos_url)
+        dc.add_file("foo").modify()
+        dc.change_prop("bzr:revision-id:v3-none", 
                             "2 myrevid\n")
-        self.client_commit("dc", "My Message")
-        self.build_tree({'dc/foo': "someotherdata"})
-        self.client_set_prop("dc", "bzr:revision-id:v3-none", 
+        dc.close()
+
+        dc = self.get_commit_editor(repos_url)
+        dc.open_file("foo").modify()
+        dc.change_prop("bzr:revision-id:v3-none", 
                             "2 myrevid\n3 mysecondrevid\n")
-        self.client_commit("dc", "My Message")
+        dc.close()
+
         branch = Branch.open(repos_url)
         self.assertEquals(3, branch.revision_id_to_revno("mysecondrevid"))
         self.assertEquals(2, branch.revision_id_to_revno("myrevid"))
 
     def test_get_nick_none(self):
-        repos_url = self.make_client('a', 'dc')
+        repos_url = self.make_repository('a')
 
-        self.build_tree({'dc/foo': "data"})
-        self.client_add("dc/foo")
-        self.client_commit("dc", "My Message")
+        dc = self.get_commit_editor(repos_url)
+        dc.add_file("foo").modify()
+        dc.close()
 
-        branch = Branch.open("svn+"+repos_url)
+        branch = Branch.open(repos_url)
 
-        self.assertIs(None, branch.nick)
+        self.assertEquals("a", branch.nick)
 
     def test_get_nick_path(self):
-        repos_url = self.make_client('a', 'dc')
+        repos_url = self.make_repository('a')
 
-        self.build_tree({'dc/trunk': None})
-        self.client_add("dc/trunk")
-        self.client_commit("dc", "My Message")
+        dc = self.get_commit_editor(repos_url)
+        dc.add_dir("trunk")
+        dc.close()
 
-        branch = Branch.open("svn+"+repos_url+"/trunk")
+        branch = Branch.open(repos_url+"/trunk")
 
         self.assertEqual("trunk", branch.nick)
 
     def test_get_revprops(self):
-        repos_url = self.make_client('a', 'dc')
+        repos_url = self.make_repository('a')
 
-        self.build_tree({'dc/foo': "data"})
-        self.client_add("dc/foo")
-        self.client_set_prop("dc", "bzr:revision-info", 
+        dc = self.get_commit_editor(repos_url)
+        dc.add_file("foo").modify()
+        dc.change_prop("bzr:revision-info", 
                 "properties: \n\tbranch-nick: mybranch\n")
-        self.client_commit("dc", "My Message")
+        dc.close()
 
-        branch = Branch.open("svn+"+repos_url)
+        branch = Branch.open(repos_url)
 
         rev = branch.repository.get_revision(branch.last_revision())
 
@@ -501,44 +634,50 @@ foohosts""")
         newbranch.lock_read()
         tree = newbranch.repository.revision_tree(oldbranch.generate_revision_id(7))
 
-        weave = newbranch.repository.weave_store.get_weave(
-            tree.inventory.path2id("hosts"),
-            newbranch.repository.get_transaction())
+        host_fileid = tree.inventory.path2id("hosts")
 
-        self.assertEqual(set([
+        self.assertVersionsPresentEquals(newbranch.repository.texts, 
+                                        host_fileid, [
             oldbranch.generate_revision_id(6),
-            oldbranch.generate_revision_id(7)]),
-                          set(weave.versions()))
+            oldbranch.generate_revision_id(7)])
         newbranch.unlock()
  
 
     def test_fetch_odd(self):
-        repos_url = self.make_client('d', 'dc')
+        repos_url = self.make_repository('d')
 
-        self.build_tree({'dc/trunk': None, 
-                         'dc/trunk/hosts': 'hej1'})
-        self.client_add("dc/trunk")
-        self.client_commit("dc", "created trunk and added hosts") #1
+        dc = self.get_commit_editor(repos_url)
+        trunk = dc.add_dir("trunk")
+        trunk.add_file("trunk/hosts").modify()
+        dc.close()
 
-        self.build_tree({'dc/trunk/hosts': 'hej2'})
-        self.client_commit("dc", "rev 2") #2
+        dc = self.get_commit_editor(repos_url)
+        trunk = dc.open_dir("trunk")
+        trunk.open_file("trunk/hosts").modify()
+        dc.close()
 
-        self.build_tree({'dc/trunk/hosts': 'hej3'})
-        self.client_commit("dc", "rev 3") #3
+        dc = self.get_commit_editor(repos_url)
+        dc.open_file("trunk/hosts").modify()
+        dc.close()
 
-        self.build_tree({'dc/branches': None})
-        self.client_add("dc/branches")
-        self.client_commit("dc", "added branches") #4
+        dc = self.get_commit_editor(repos_url)
+        dc.add_dir("branches")
+        dc.close()
 
-        self.client_copy("dc/trunk", "dc/branches/foobranch")
-        self.client_commit("dc", "added branch foobranch") #5
+        dc = self.get_commit_editor(repos_url)
+        branches = dc.open_dir("branches")
+        branches.add_dir("branches/foobranch", "trunk")
+        dc.close()
 
-        self.build_tree({'dc/branches/foobranch/hosts': 'foohosts'})
-        self.client_commit("dc", "foohosts") #6
+        dc = self.get_commit_editor(repos_url)
+        branches = dc.open_dir("branches")
+        foobranch = branches.open_dir("branches/foobranch")
+        foobranch.open_file("branches/foobranch/hosts").modify()
+        dc.close()
 
         os.mkdir("new")
 
-        url = "svn+"+repos_url+"/branches/foobranch"
+        url = repos_url+"/branches/foobranch"
         mutter('open %r' % url)
         olddir = BzrDir.open(url)
 
@@ -552,42 +691,49 @@ foohosts""")
              oldbranch.generate_revision_id(6))
         transaction = newbranch.repository.get_transaction()
         newbranch.repository.lock_read()
-        weave = newbranch.repository.weave_store.get_weave(
-                tree.inventory.path2id("hosts"), transaction)
+        texts = newbranch.repository.texts
+        host_fileid = tree.inventory.path2id("hosts")
         mapping = BzrSvnMappingv3FileProps(TrunkBranchingScheme())
-        self.assertEqual(set([
+        self.assertVersionsPresentEquals(texts, host_fileid, [
             mapping.generate_revision_id(uuid, 1, "trunk"),
             mapping.generate_revision_id(uuid, 2, "trunk"),
             mapping.generate_revision_id(uuid, 3, "trunk"),
-            oldbranch.generate_revision_id(6)]),
-                          set(weave.versions()))
+            oldbranch.generate_revision_id(6)])
         newbranch.repository.unlock()
 
+    def assertVersionsPresentEquals(self, texts, fileid, versions):
+        self.assertEqual(set([(fileid, v) for v in versions]),
+            set(filter(lambda (fid, rid): fid == fileid, texts.keys())))
+
     def test_check(self):
-        self.make_client('d', 'dc')
+        self.make_repository('d')
         branch = Branch.open('d')
         result = branch.check()
         self.assertEqual(branch, result.branch) 
  
     def test_generate_revision_id(self):
-        self.make_client('d', 'dc')
-        self.build_tree({'dc/bla/bloe': None})
-        self.client_add("dc/bla")
-        self.client_commit("dc", "bla")
+        repos_url = self.make_repository('d')
+
+        dc = self.get_commit_editor(repos_url)
+        bla = dc.add_dir("bla")
+        bla.add_dir("bla/bloe")
+        dc.close()
+
         branch = Branch.open('d')
         self.assertEqual("svn-v3-none:%s::1" % (branch.repository.uuid),  branch.generate_revision_id(1))
 
     def test_create_checkout(self):
-        repos_url = self.make_client('d', 'dc')
+        repos_url = self.make_repository('d')
 
-        self.build_tree({'dc/trunk': None, 'dc/trunk/hosts': 'hej1'})
-        self.client_add("dc/trunk")
-        self.client_commit("dc", "created trunk and added hosts") #1
+        dc = self.get_commit_editor(repos_url)
+        trunk = dc.add_dir("trunk")
+        trunk.add_file("trunk/hosts").modify()
+        dc.close()
 
-        url = "svn+"+repos_url+"/trunk"
+        url = repos_url+"/trunk"
         oldbranch = Branch.open(url)
 
-        newtree = self.create_checkout(oldbranch, "e")
+        newtree = oldbranch.create_checkout("e")
         self.assertTrue(newtree.branch.repository.has_revision(
            oldbranch.generate_revision_id(1)))
 
@@ -595,38 +741,36 @@ foohosts""")
         self.assertFalse(os.path.exists("e/.svn"))
 
     def test_create_checkout_lightweight(self):
-        repos_url = self.make_client('d', 'dc')
+        repos_url = self.make_repository('d')
 
-        self.build_tree({'dc/trunk': None, 
-                         'dc/trunk/hosts': 'hej1'})
-        self.client_add("dc/trunk")
-        self.client_commit("dc", "created trunk and added hosts") #1
+        dc = self.get_commit_editor(repos_url)
+        trunk = dc.add_dir("trunk")
+        trunk.add_file("trunk/hosts")
+        dc.close()
 
-        url = "svn+"+repos_url+"/trunk"
-        oldbranch = Branch.open(url)
-
-        newtree = self.create_checkout(oldbranch, "e", lightweight=True)
+        oldbranch = Branch.open(repos_url+"/trunk")
+        newtree = oldbranch.create_checkout("e", lightweight=True)
         self.assertEqual(oldbranch.generate_revision_id(1), newtree.base_revid)
         self.assertTrue(os.path.exists("e/.svn"))
         self.assertFalse(os.path.exists("e/.bzr"))
 
     def test_create_checkout_lightweight_stop_rev(self):
-        repos_url = self.make_client('d', 'dc')
+        repos_url = self.make_repository('d')
 
-        self.build_tree({'dc/trunk': None, 
-                         'dc/trunk/hosts': 'hej1'})
-        self.client_add("dc/trunk")
-        self.client_commit("dc", "created trunk and added hosts") #1
-        self.client_update("dc")
-        
-        self.build_tree({'dc/trunk/hosts': 'bloe'})
-        self.client_commit("dc", "added another revision")
-        self.client_update("dc")
+        dc = self.get_commit_editor(repos_url)
+        trunk = dc.add_dir("trunk")
+        trunk.add_file("trunk/hosts").modify()
+        dc.close()
 
-        url = "svn+"+repos_url+"/trunk"
+        dc = self.get_commit_editor(repos_url)
+        trunk = dc.open_dir("trunk")
+        trunk.open_file("trunk/hosts").modify()
+        dc.close()
+
+        url = repos_url+"/trunk"
         oldbranch = Branch.open(url)
 
-        newtree = self.create_checkout(oldbranch, "e", revision_id=
+        newtree = oldbranch.create_checkout("e", revision_id=
            oldbranch.generate_revision_id(1), lightweight=True)
         self.assertEqual(oldbranch.generate_revision_id(1),
            newtree.base_revid)
@@ -634,13 +778,14 @@ foohosts""")
         self.assertFalse(os.path.exists("e/.bzr"))
 
     def test_fetch_branch(self):
-        self.make_client('d', 'sc')
+        repos_url = self.make_client('d', 'sc')
 
-        self.build_tree({'sc/foo/bla': "data"})
-        self.client_add("sc/foo")
-        self.client_commit("sc", "foo")
+        sc = self.get_commit_editor(repos_url)
+        foo = sc.add_dir("foo")
+        foo.add_file("foo/bla").modify()
+        sc.close()
 
-        olddir = self.open_checkout_bzrdir("sc")
+        olddir = BzrDir.open("sc")
 
         os.mkdir("dc")
         
@@ -653,15 +798,20 @@ foohosts""")
     def test_fetch_dir_upgrade(self):
         repos_url = self.make_client('d', 'sc')
 
-        self.build_tree({'sc/trunk/mylib/bla': "data", "sc/branches": None})
-        self.client_add("sc/trunk")
-        self.client_add("sc/branches")
-        self.client_commit("sc", "foo")
+        sc = self.get_commit_editor(repos_url)
+        trunk = sc.add_dir("trunk")
+        mylib = trunk.add_dir("trunk/mylib")
+        mylib.add_file("trunk/mylib/bla").modify()
+        sc.add_dir("branches")
+        sc.close()
 
-        self.client_copy("sc/trunk/mylib", "sc/branches/abranch")
-        self.client_commit("sc", "Promote mylib")
+        sc = self.get_commit_editor(repos_url)
+        branches = sc.open_dir("branches")
+        branches.add_dir("branches/abranch", "trunk/mylib")
+        sc.close()
 
-        olddir = self.open_checkout_bzrdir("sc/branches/abranch")
+        self.client_update('sc')
+        olddir = BzrDir.open("sc/branches/abranch")
 
         os.mkdir("dc")
         
@@ -674,15 +824,20 @@ foohosts""")
     def test_fetch_branch_downgrade(self):
         repos_url = self.make_client('d', 'sc')
 
-        self.build_tree({'sc/trunk': None, "sc/branches/abranch/bla": 'foo'})
-        self.client_add("sc/trunk")
-        self.client_add("sc/branches")
-        self.client_commit("sc", "foo")
+        sc = self.get_commit_editor(repos_url)
+        sc.add_dir("trunk")
+        branches = sc.add_dir("branches")
+        abranch = branches.add_dir("branches/abranch")
+        abranch.add_file("branches/abranch/bla").modify()
+        sc.close()
 
-        self.client_copy("sc/branches/abranch", "sc/trunk/mylib")
-        self.client_commit("sc", "Demote mylib")
+        sc = self.get_commit_editor(repos_url)
+        trunk = sc.open_dir("trunk")
+        sc.add_dir("trunk/mylib", "branches/abranch")
+        sc.close()
 
-        olddir = self.open_checkout_bzrdir("sc/trunk")
+        self.client_update('sc')
+        olddir = BzrDir.open("sc/trunk")
 
         os.mkdir("dc")
         
@@ -697,14 +852,15 @@ foohosts""")
     def test_ghost_workingtree(self):
         # Looks like bazaar has trouble creating a working tree of a 
         # revision that has ghost parents
-        self.make_client('d', 'sc')
+        repos_url = self.make_client('d', 'sc')
 
-        self.build_tree({'sc/foo/bla': "data"})
-        self.client_add("sc/foo")
-        self.client_set_prop("sc", "bzr:ancestry:v3-none", "some-ghost\n")
-        self.client_commit("sc", "foo")
+        sc = self.get_commit_editor(repos_url)
+        foo = sc.add_dir("foo")
+        foo.add_file("foo/bla").modify()
+        sc.change_prop("bzr:ancestry:v3-none", "some-ghost\n")
+        sc.close()
 
-        olddir = self.open_checkout_bzrdir("sc")
+        olddir = BzrDir.open("sc")
 
         os.mkdir("dc")
         
