@@ -16,12 +16,10 @@
 
 """Branch property access tests."""
 
-from bzrlib.errors import NoSuchRevision
-
-from tests import TestCaseWithSubversionRepository
-from branchprops import PathPropertyProvider
-from logwalker import LogWalker, CachingLogWalker
-from transport import SvnRaTransport
+from bzrlib.plugins.svn.tests import TestCaseWithSubversionRepository
+from bzrlib.plugins.svn.branchprops import PathPropertyProvider
+from bzrlib.plugins.svn.logwalker import LogWalker, CachingLogWalker
+from bzrlib.plugins.svn.transport import SvnRaTransport
 
 class TestBranchProps(TestCaseWithSubversionRepository):
     def setUp(self):
@@ -31,12 +29,15 @@ class TestBranchProps(TestCaseWithSubversionRepository):
         return CachingLogWalker(LogWalker(transport=transport))
 
     def test_get_old_properties(self):
-        repos_url = self.make_client('d', 'dc')
-        self.client_set_prop("dc", "myprop", "data")
-        self.client_commit("dc", "My Message")
-        self.build_tree({'dc/foo': "bla"})
-        self.client_add("dc/foo")
-        self.client_commit("dc", "My Message")
+        repos_url = self.make_repository('d')
+
+        dc = self.get_commit_editor(repos_url)
+        dc.change_prop("myprop", "data")
+        dc.close()
+
+        dc = self.get_commit_editor(repos_url)
+        dc.add_file("foo").modify()
+        dc.close()
 
         logwalk = self.get_log_walker(transport=SvnRaTransport(repos_url))
 
@@ -45,9 +46,11 @@ class TestBranchProps(TestCaseWithSubversionRepository):
         self.assertEqual("data", bp.get_properties("", 2)["myprop"])
 
     def test_get_properties(self):
-        repos_url = self.make_client('d', 'dc')
-        self.client_set_prop("dc", "myprop", "data")
-        self.client_commit("dc", "My Message")
+        repos_url = self.make_repository('d')
+
+        dc = self.get_commit_editor(repos_url)
+        dc.change_prop("myprop", "data")
+        dc.close()
 
         transport = SvnRaTransport(repos_url)
         logwalk = self.get_log_walker(transport=transport)
@@ -61,16 +64,19 @@ class TestBranchProps(TestCaseWithSubversionRepository):
         self.assertTrue("svn:entry:committed-date" in props)
 
     def test_get_changed_properties(self):
-        repos_url = self.make_client('d', 'dc')
-        self.client_set_prop("dc", "myprop", "data\n")
-        self.client_commit("dc", "My Message")
-        self.client_update("dc")
-        self.client_set_prop("dc", "myprop", "newdata\n")
-        self.client_commit("dc", "My Message")
-        self.client_update("dc")
-        self.client_set_prop("dc", "myp2", "newdata\n")
-        self.client_commit("dc", "My Message")
-        self.client_update("dc")
+        repos_url = self.make_repository('d')
+
+        dc = self.get_commit_editor(repos_url)
+        dc.change_prop("myprop", "data\n")
+        dc.close()
+
+        dc = self.get_commit_editor(repos_url)
+        dc.change_prop("myprop", "newdata\n")
+        dc.close()
+
+        dc = self.get_commit_editor(repos_url)
+        dc.change_prop("myp2", "newdata\n")
+        dc.close()
 
         logwalk = self.get_log_walker(transport=SvnRaTransport(repos_url))
 
