@@ -21,18 +21,17 @@ from bzrlib.bzrdir import BzrDir
 from bzrlib.errors import NoRepositoryPresent, UninitializableFormat
 from bzrlib.tests import TestCase
 
-from convert import SvnConverter
 from bzrlib.plugins.svn.workingtree import SvnWorkingTreeFormat
 from bzrlib.plugins.svn.format import SvnWorkingTreeDirFormat
-from tests import TestCaseWithSubversionRepository
+from bzrlib.plugins.svn.tests import TestCaseWithSubversionRepository
 
 class TestWorkingTreeFormat(TestCase):
     def setUp(self):
         super(TestWorkingTreeFormat, self).setUp()
-        self.format = SvnWorkingTreeFormat()
+        self.format = SvnWorkingTreeFormat(4)
 
     def test_get_format_desc(self):
-        self.assertEqual("Subversion Working Copy", 
+        self.assertEqual("Subversion Working Copy Version 4", 
                          self.format.get_format_description())
 
     def test_initialize(self):
@@ -58,32 +57,32 @@ class TestCheckoutFormat(TestCase):
 class TestCheckout(TestCaseWithSubversionRepository):
     def test_not_for_writing(self):
         self.make_client("d", "dc")
-        x = self.create_branch_convenience("dc/foo")
+        x = BzrDir.create_branch_convenience("dc/foo")
         self.assertFalse(hasattr(x.repository, "uuid"))
 
     def test_open_repository(self):
         self.make_client("d", "dc")
-        x = self.open_checkout_bzrdir("dc")
+        x = BzrDir.open("dc")
         self.assertRaises(NoRepositoryPresent, x.open_repository)
 
     def test_find_repository(self):
         self.make_client("d", "dc")
-        x = self.open_checkout_bzrdir("dc")
+        x = BzrDir.open("dc")
         self.assertRaises(NoRepositoryPresent, x.find_repository)
 
     def test__find_repository(self):
         self.make_client("d", "dc")
-        x = self.open_checkout_bzrdir("dc")
+        x = BzrDir.open("dc")
         self.assertTrue(hasattr(x._find_repository(), "uuid"))
 
     def test_needs_format_conversion_default(self):
         self.make_client("d", "dc")
-        x = self.open_checkout_bzrdir("dc")
+        x = BzrDir.open("dc")
         self.assertTrue(x.needs_format_conversion())
 
     def test_needs_format_conversion_self(self):
         self.make_client("d", "dc")
-        x = self.open_checkout_bzrdir("dc")
+        x = BzrDir.open("dc")
         self.assertFalse(x.needs_format_conversion(SvnWorkingTreeDirFormat()))
         
     def test_checkout_checkout(self):
@@ -94,8 +93,11 @@ class TestCheckout(TestCaseWithSubversionRepository):
 
     def test_checkout_branch(self):
         repos_url = self.make_client("d", "dc")
-        self.build_tree({"dc/trunk": None})
-        self.client_add("dc/trunk")
-        self.client_commit("dc", "msg")
-        x = self.open_checkout_bzrdir("dc/trunk")
+
+        dc = self.get_commit_editor(repos_url)
+        dc.add_dir("trunk")
+        dc.close()
+
+        self.client_update("dc")
+        x = BzrDir.open("dc/trunk")
         self.assertEquals(repos_url+"/trunk", x.open_branch().base)
