@@ -17,9 +17,9 @@
 """Config tests."""
 
 from bzrlib.branch import Branch
-from config import SvnRepositoryConfig, BranchConfig
-from mapping3.scheme import TrunkBranchingScheme
-from tests import TestCaseWithSubversionRepository
+from bzrlib.plugins.svn.config import SvnRepositoryConfig, BranchConfig
+from bzrlib.plugins.svn.mapping3.scheme import TrunkBranchingScheme
+from bzrlib.plugins.svn.tests import TestCaseWithSubversionRepository
 
 from bzrlib.tests import TestCaseInTempDir
 
@@ -48,7 +48,7 @@ class ReposConfigTests(TestCaseInTempDir):
 
     def test_get_scheme_set(self):
         c = SvnRepositoryConfig("blabla2")
-        c.set_branching_scheme(TrunkBranchingScheme())
+        c.set_branching_scheme(TrunkBranchingScheme(), None)
         self.assertEquals("trunk0", str(c.get_branching_scheme()))
 
     def test_get_scheme_mandatory_none(self):
@@ -57,9 +57,9 @@ class ReposConfigTests(TestCaseInTempDir):
 
     def test_get_scheme_mandatory_set(self):
         c = SvnRepositoryConfig("blabla3")
-        c.set_branching_scheme(TrunkBranchingScheme(), mandatory=True)
+        c.set_branching_scheme(TrunkBranchingScheme(), None, mandatory=True)
         self.assertEquals(True, c.branching_scheme_is_mandatory())
-        c.set_branching_scheme(TrunkBranchingScheme(), mandatory=False)
+        c.set_branching_scheme(TrunkBranchingScheme(), None, mandatory=False)
         self.assertEquals(False, c.branching_scheme_is_mandatory())
 
     def test_override_revprops(self):
@@ -70,7 +70,7 @@ class ReposConfigTests(TestCaseInTempDir):
         c.set_user_option("override-svn-revprops", "False")
         self.assertEquals([], c.get_override_svn_revprops())
         c.set_user_option("override-svn-revprops", ["svn:author", "svn:date"])
-        self.assertEquals(["svn:author","svn:date"], c.get_override_svn_revprops())
+        self.assertEquals(["svn:author", "svn:date"], c.get_override_svn_revprops())
         c.set_user_option("override-svn-revprops", ["svn:author"])
         self.assertEquals(["svn:author"], c.get_override_svn_revprops())
 
@@ -82,14 +82,6 @@ class ReposConfigTests(TestCaseInTempDir):
         c.set_user_option("append_revisions_only", "False")
         self.assertEquals(False, c.get_append_revisions_only())
 
-    def test_set_revprops(self):
-        c = SvnRepositoryConfig("blabla2")
-        self.assertEquals(None, c.get_set_revprops())
-        c.set_user_option("set-revprops", "True")
-        self.assertEquals(True, c.get_set_revprops())
-        c.set_user_option("set-revprops", "False")
-        self.assertEquals(False, c.get_set_revprops())
-
     def test_log_strip_trailing_newline(self):
         c = SvnRepositoryConfig("blabla3")
         self.assertEquals(False, c.get_log_strip_trailing_newline())
@@ -110,7 +102,7 @@ class ReposConfigTests(TestCaseInTempDir):
 class BranchConfigTests(TestCaseWithSubversionRepository):
     def setUp(self):
         super(BranchConfigTests, self).setUp()
-        self.repos_url = self.make_client("d", "dc")
+        self.repos_url = self.make_repository("d")
         self.config = BranchConfig(Branch.open(self.repos_url))
 
     def test_set_option(self):
