@@ -22,16 +22,15 @@ from bzrlib.errors import NotBranchError, NoSuchFile, IncompatibleRepositories
 from bzrlib.urlutils import local_path_to_url
 from bzrlib.repository import Repository
 from bzrlib.tests import TestCaseInTempDir
-from bzrlib.trace import mutter
 
 import os, sys
-from convert import convert_repository, NotDumpFile, load_dumpfile
-from format import get_rich_root_format
-from mapping3 import set_branching_scheme
-from mapping3.scheme import TrunkBranchingScheme, NoBranchingScheme
-from tests import TestCaseWithSubversionRepository
 
-import svn.repos
+from bzrlib.plugins.svn import repos
+from bzrlib.plugins.svn.convert import convert_repository, NotDumpFile, load_dumpfile
+from bzrlib.plugins.svn.format import get_rich_root_format
+from bzrlib.plugins.svn.mapping3 import set_branching_scheme
+from bzrlib.plugins.svn.mapping3.scheme import TrunkBranchingScheme, NoBranchingScheme
+from bzrlib.plugins.svn.tests import TestCaseWithSubversionRepository
 
 class TestLoadDumpfile(TestCaseInTempDir):
     def test_loaddumpfile(self):
@@ -52,10 +51,9 @@ V 27
 PROPS-END
 """)
         load_dumpfile(dumpfile, "d")
-        repos = svn.repos.open("d")
-        fs = svn.repos.fs(repos)
+        fs = repos.Repository("d").fs()
         self.assertEqual("6987ef2d-cd6b-461f-9991-6f1abef3bd59", 
-                svn.fs.get_uuid(fs))
+                fs.get_uuid())
 
     def test_loaddumpfile_invalid(self):
         dumpfile = os.path.join(self.test_dir, "dumpfile")
@@ -66,13 +64,23 @@ PROPS-END
 class TestConversion(TestCaseWithSubversionRepository):
     def setUp(self):
         super(TestConversion, self).setUp()
-        self.repos_url = self.make_client('d', 'dc')
-        self.build_tree({'dc/trunk/file': 'data', 'dc/branches/abranch/anotherfile': 'data2'})
-        self.client_add("dc/trunk")
-        self.client_add("dc/branches")
-        self.client_commit("dc", "create repos")
-        self.build_tree({'dc/trunk/file': 'otherdata'})
-        self.client_commit("dc", "change")
+        self.repos_url = self.make_repository('d')
+
+        dc = self.get_commit_editor()
+        t = dc.add_dir("trunk")
+        t.add_file("trunk/file").modify("data")
+        bs = dc.add_dir("branches")
+        ab = bs.add_dir("branches/abranch")
+        ab.add_file("branches/abranch/anotherfile").modify("data2")
+        dc.close()
+
+        dc = self.get_commit_editor()
+        t = dc.open_dir("trunk")
+        t.open_file("trunk/file").modify("otherdata")
+        dc.close()
+
+    def get_commit_editor(self):
+        return super(TestConversion, self).get_commit_editor(self.repos_url)
 
     def test_sets_parent_urls(self):
         convert_repository(Repository.open(self.repos_url), "e", 
@@ -84,12 +92,17 @@ class TestConversion(TestCaseWithSubversionRepository):
                 Branch.open("e/branches/abranch").get_parent())
 
     def test_fetch_alive(self):
-        self.build_tree({'dc/branches/somebranch/somefile': 'data'})
-        self.client_add("dc/branches/somebranch")
-        self.client_commit("dc", "add a branch")
-        self.client_update("dc")
-        self.client_delete("dc/branches/somebranch")
-        self.client_commit("dc", "remove branch")
+        dc = self.get_commit_editor()
+        bs = dc.open_dir("branches")
+        sb = bs.add_dir("branches/somebranch")
+        sb.add_file("branches/somebranch/somefile").modify('data')
+        dc.close()
+
+        dc = self.get_commit_editor()
+        bs = dc.open_dir("branches")
+        bs.delete("branches/somebranch")
+        dc.close()
+
         oldrepos = Repository.open(self.repos_url)
         convert_repository(oldrepos, "e", 
                            TrunkBranchingScheme(), 
@@ -99,9 +112,11 @@ class TestConversion(TestCaseWithSubversionRepository):
         self.assertFalse(newrepos.has_revision(oldrepos.generate_revision_id(2, "branches/somebranch", oldrepos.get_mapping())))
 
     def test_fetch_filebranch(self):
-        self.build_tree({'dc/branches/somebranch': 'data'})
-        self.client_add("dc/branches/somebranch")
-        self.client_commit("dc", "add a branch")
+        dc = self.get_commit_editor()
+        bs = dc.open_dir("branches")
+        bs.add_file("branches/somebranch").modify('data')
+        dc.close()
+
         oldrepos = Repository.open(self.repos_url)
         convert_repository(oldrepos, "e", TrunkBranchingScheme())
         newrepos = Repository.open("e")
@@ -109,11 +124,17 @@ class TestConversion(TestCaseWithSubversionRepository):
         self.assertFalse(newrepos.has_revision(oldrepos.generate_revision_id(2, "branches/somebranch", oldrepos.get_mapping())))
 
     def test_fetch_dead(self):
-        self.build_tree({'dc/branches/somebranch/somefile': 'data'})
-        self.client_add("dc/branches/somebranch")
-        self.client_commit("dc", "add a branch")
-        self.client_delete("dc/branches/somebranch")
-        self.client_commit("dc", "remove branch")
+        dc = self.get_commit_editor()
+        bs = dc.open_dir("branches")
+        sb = bs.add_dir("branches/somebranch")
+        sb.add_file("branches/somebranch/somefile").modify('data')
+        dc.close()
+
+        dc = self.get_commit_editor()
+        bs = dc.open_dir("branches")
+        bs.delete("branches/somebranch")
+        dc.close()
+
         oldrepos = Repository.open(self.repos_url)
         convert_repository(oldrepos, "e", TrunkBranchingScheme(), 
                            all=True, create_shared_repo=True)
@@ -122,12 +143,18 @@ class TestConversion(TestCaseWithSubversionRepository):
             oldrepos.generate_revision_id(3, "branches/somebranch", oldrepos.get_mapping())))
 
     def test_fetch_filter(self):
-        self.build_tree({'dc/branches/somebranch/somefile': 'data',
-                         'dc/branches/anotherbranch/somefile': 'data'})
-        self.client_add("dc/branches/somebranch")
-        self.client_commit("dc", "add a branch")
-        self.client_add("dc/branches/anotherbranch")
-        self.client_commit("dc", "add another branch")
+        dc = self.get_commit_editor()
+        branches = dc.open_dir("branches")
+        dc.add_dir("branches/somebranch")
+        dc.add_file("branches/somebranch/somefile").modify('data')
+        dc.close()
+
+        dc = self.get_commit_editor()
+        branches = dc.open_dir("branches")
+        ab = branches.add_dir("branches/anotherbranch")
+        ab.add_file("branches/anotherbranch/somefile").modify('data')
+        dc.close()
+
         oldrepos = Repository.open(self.repos_url)
         convert_repository(oldrepos, "e", TrunkBranchingScheme(), 
             create_shared_repo=True,
@@ -140,7 +167,7 @@ class TestConversion(TestCaseWithSubversionRepository):
         dir = BzrDir.create("e", format=get_rich_root_format())
         dir.create_repository(shared=True)
 
-        convert_repository(Repository.open("svn+"+self.repos_url), "e", 
+        convert_repository(Repository.open(self.repos_url), "e", 
                 TrunkBranchingScheme(), create_shared_repo=True)
 
         self.assertTrue(Repository.open("e").is_shared())
@@ -148,33 +175,52 @@ class TestConversion(TestCaseWithSubversionRepository):
     def test_shared_import_continue_remove(self):
         convert_repository(Repository.open(self.repos_url), "e", 
                 TrunkBranchingScheme(), create_shared_repo=True)
-        self.client_update("dc")
-        self.client_delete("dc/trunk")
-        self.client_commit("dc", "blafoo")
-        self.build_tree({'dc/trunk/file': 'otherdata'})
-        self.client_add("dc/trunk")
-        self.client_commit("dc", "change")
+
+        dc = self.get_commit_editor()
+        dc.delete("trunk")
+        dc.close()
+
+        dc = self.get_commit_editor()
+        trunk = dc.add_dir("trunk")
+        trunk.add_file("trunk/file").modify()
+        dc.close()
+
         convert_repository(Repository.open(self.repos_url), "e", 
                            TrunkBranchingScheme(), create_shared_repo=True)
 
+    def test_shared_import_remove_nokeep(self):
+        convert_repository(Repository.open(self.repos_url), "e", 
+                TrunkBranchingScheme(), create_shared_repo=True)
+
+        dc = self.get_commit_editor()
+        dc.delete("trunk")
+        dc.close()
+
+        self.assertTrue(os.path.exists("e/trunk"))
+
+        convert_repository(Repository.open(self.repos_url), "e", 
+                           TrunkBranchingScheme(), create_shared_repo=True)
+
+        self.assertFalse(os.path.exists("e/trunk"))
+
     def test_shared_import_continue_with_wt(self):
-        convert_repository(Repository.open("svn+"+self.repos_url), "e", 
+        convert_repository(Repository.open(self.repos_url), "e", 
                 TrunkBranchingScheme(), working_trees=True)
-        convert_repository(Repository.open("svn+"+self.repos_url), "e", 
+        convert_repository(Repository.open(self.repos_url), "e", 
                 TrunkBranchingScheme(), working_trees=True)
 
     def test_shared_import_nonescheme_empty(self):
         dir = BzrDir.create("e", format=get_rich_root_format())
         dir.create_repository(shared=True)
 
-        convert_repository(Repository.open("svn+"+self.repos_url), "e", 
+        convert_repository(Repository.open(self.repos_url), "e", 
                 NoBranchingScheme(), create_shared_repo=True)
 
     def test_shared_import_with_wt(self):
         dir = BzrDir.create("e", format=get_rich_root_format())
         dir.create_repository(shared=True)
 
-        convert_repository(Repository.open("svn+"+self.repos_url), "e", 
+        convert_repository(Repository.open(self.repos_url), "e", 
                 TrunkBranchingScheme(), create_shared_repo=True, 
                 working_trees=True)
 
@@ -185,7 +231,7 @@ class TestConversion(TestCaseWithSubversionRepository):
         dir = BzrDir.create("e", format=get_rich_root_format())
         dir.create_repository(shared=True)
 
-        convert_repository(Repository.open("svn+"+self.repos_url), "e", 
+        convert_repository(Repository.open(self.repos_url), "e", 
                 TrunkBranchingScheme(), create_shared_repo=True, 
                 working_trees=False)
 
@@ -202,20 +248,22 @@ class TestConversion(TestCaseWithSubversionRepository):
                 working_trees=False))
 
     def test_shared_import_continue_branch(self):
-        oldrepos = Repository.open("svn+"+self.repos_url)
+        oldrepos = Repository.open(self.repos_url)
         convert_repository(oldrepos, "e", 
                 TrunkBranchingScheme(), create_shared_repo=True)
 
         mapping = oldrepos.get_mapping()
 
-        self.build_tree({'dc/trunk/file': 'foodata'})
-        self.client_commit("dc", "msg")
+        dc = self.get_commit_editor()
+        trunk = dc.open_dir("trunk")
+        trunk.open_file("trunk/file").modify()
+        dc.close()
 
         self.assertEqual(
                 Repository.open(self.repos_url).generate_revision_id(2, "trunk", mapping), 
                 Branch.open("e/trunk").last_revision())
 
-        convert_repository(Repository.open("svn+"+self.repos_url), "e", 
+        convert_repository(Repository.open(self.repos_url), "e", 
                 TrunkBranchingScheme(), create_shared_repo=True)
 
         self.assertEqual(Repository.open(self.repos_url).generate_revision_id(3, "trunk", mapping), 
@@ -223,21 +271,21 @@ class TestConversion(TestCaseWithSubversionRepository):
 
  
     def test_shared_import(self):
-        convert_repository(Repository.open("svn+"+self.repos_url), "e", 
+        convert_repository(Repository.open(self.repos_url), "e", 
                 TrunkBranchingScheme(), create_shared_repo=True)
 
         self.assertTrue(Repository.open("e").is_shared())
     
     def test_simple(self):
-        convert_repository(Repository.open("svn+"+self.repos_url), os.path.join(self.test_dir, "e"), TrunkBranchingScheme())
+        convert_repository(Repository.open(self.repos_url), os.path.join(self.test_dir, "e"), TrunkBranchingScheme())
         self.assertTrue(os.path.isdir(os.path.join(self.test_dir, "e", "trunk")))
         self.assertTrue(os.path.isdir(os.path.join(self.test_dir, "e", "branches", "abranch")))
 
     def test_convert_to_nonexistant(self):
-        self.assertRaises(NoSuchFile, convert_repository, Repository.open("svn+"+self.repos_url), os.path.join(self.test_dir, "e", "foo", "bar"), TrunkBranchingScheme())
+        self.assertRaises(NoSuchFile, convert_repository, Repository.open(self.repos_url), os.path.join(self.test_dir, "e", "foo", "bar"), TrunkBranchingScheme())
 
     def test_notshared_import(self):
-        convert_repository(Repository.open("svn+"+self.repos_url), "e", 
+        convert_repository(Repository.open(self.repos_url), "e", 
                            TrunkBranchingScheme(), create_shared_repo=False)
 
         self.assertRaises(NotBranchError, Repository.open, "e")
