@@ -16,40 +16,121 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-from bzrlib.branch import Branch, BranchReferenceFormat
-from bzrlib.bzrdir import BzrDir, BzrDirFormat
+from bzrlib.branch import Branch
+from bzrlib.bzrdir import BzrDir
 from bzrlib.errors import AlreadyBranchError, BzrError, DivergedBranches
-from bzrlib.inventory import Inventory
 from bzrlib.merge import Merger, Merge3Merger
 from bzrlib.osutils import has_symlinks
 from bzrlib.progress import DummyProgress
 from bzrlib.repository import Repository
-from bzrlib.tests import KnownFailure, TestCaseWithTransport
 from bzrlib.trace import mutter
-from bzrlib.workingtree import WorkingTree
 
 import os
-import format
-import svn.core
-from bzrlib.plugins.svn.errors import ChangesRootLHSHistory, MissingPrefix
-from time import sleep
-from commit import push
-from mapping import SVN_PROP_BZR_REVISION_ID
-from tests import TestCaseWithSubversionRepository
+
+from bzrlib.plugins.svn import core, format, ra
+from bzrlib.plugins.svn.errors import MissingPrefix
+from bzrlib.plugins.svn.commit import push, dpush
+from bzrlib.plugins.svn.mapping import SVN_PROP_BZR_REVISION_ID
+from bzrlib.plugins.svn.tests import TestCaseWithSubversionRepository
+
+class TestDPush(TestCaseWithSubversionRepository):
+    def setUp(self):
+        super(TestDPush, self).setUp()
+        self.repos_url = self.make_repository('d')
+
+        dc = self.commit_editor()
+        foo = dc.add_dir("foo")
+        foo.add_file("foo/bla").modify("data")
+        dc.close()
+
+        self.svndir = BzrDir.open(self.repos_url)
+        os.mkdir("dc")
+        self.bzrdir = self.svndir.sprout("dc")
+
+    def commit_editor(self):
+        return self.get_commit_editor(self.repos_url)
+
+    def test_change_single(self):
+        self.build_tree({'dc/foo/bla': 'other data'})
+        wt = self.bzrdir.open_workingtree()
+        newid = wt.commit(message="Commit from Bzr")
+
+        revid_map = dpush(self.svndir.open_branch(), self.bzrdir.open_branch())
+
+        self.assertEquals([newid], revid_map.keys())
+
+        c = ra.RemoteAccess(self.repos_url)
+        (entries, fetch_rev, props) = c.get_dir("", c.get_latest_revnum())
+        self.assertEquals(set(['svn:entry:committed-rev', 
+            'svn:entry:last-author', 'svn:entry:uuid', 
+            'svn:entry:committed-date']), set(props.keys()))
+
+        r = self.svndir.find_repository()
+        self.assertEquals([r.generate_revision_id(
+                c.get_latest_revnum(),
+                "", 
+                r.get_mapping())], revid_map.values())
+
+    def test_change_multiple(self):
+        self.build_tree({'dc/foo/bla': 'other data'})
+        wt = self.bzrdir.open_workingtree()
+        self.build_tree({'dc/foo/bla': 'other data'})
+        newid1 = wt.commit(message="Commit from Bzr")
+        self.build_tree({'dc/foo/bla': 'yet other data'})
+        newid2 = wt.commit(message="Commit from Bzr")
+
+        revid_map = dpush(self.svndir.open_branch(), self.bzrdir.open_branch())
+
+        self.assertEquals(set([newid1, newid2]), set(revid_map.keys()))
+
+        c = ra.RemoteAccess(self.repos_url)
+        (entries, fetch_rev, props) = c.get_dir("", c.get_latest_revnum())
+        self.assertEquals(set(['svn:entry:committed-rev', 
+            'svn:entry:last-author', 'svn:entry:uuid', 
+            'svn:entry:committed-date']), set(props.keys()))
+
+        r = self.svndir.find_repository()
+        self.assertEquals(set([r.generate_revision_id(
+                rev,
+                "", 
+                r.get_mapping()) for rev in (c.get_latest_revnum()-1, c.get_latest_revnum())]), 
+                set(revid_map.values()))
+ 
+    def test_diverged(self):
+        dc = self.commit_editor()
+        foo = dc.open_dir("foo")
+        foo.add_file("foo/bar").modify("data")
+        dc.close()
+
+        svndir = BzrDir.open(self.repos_url)
+
+        self.build_tree({'dc/file': 'data'})
+        wt = self.bzrdir.open_workingtree()
+        wt.add('file')
+        wt.commit(message="Commit from Bzr")
+
+        self.assertRaises(DivergedBranches, 
+                          dpush, svndir.open_branch(),
+                          self.bzrdir.open_branch())
+
 
 class TestPush(TestCaseWithSubversionRepository):
     def setUp(self):
         super(TestPush, self).setUp()
-        self.repos_url = self.make_client('d', 'sc')
+        self.repos_url = self.make_repository('d')
 
-        self.build_tree({'sc/foo/bla': "data"})
-        self.client_add("sc/foo")
-        self.client_commit("sc", "foo")
+        dc = self.commit_editor()
+        foo = dc.add_dir("foo")
+        foo.add_file("foo/bla").modify("data")
+        dc.close()
 
-        self.svndir = BzrDir.open("sc")
+        self.svndir = BzrDir.open(self.repos_url)
         os.mkdir("dc")
         self.bzrdir = self.svndir.sprout("dc")
 
+    def commit_editor(self):
+        return self.get_commit_editor(self.repos_url)
+
     def test_empty(self):
         svnbranch = self.svndir.open_branch()
         bzrbranch = self.bzrdir.open_branch()
@@ -59,9 +140,10 @@ class TestPush(TestCaseWithSubversionRepository):
                          bzrbranch.revision_history())
 
     def test_child(self):
-        self.build_tree({'sc/foo/bar': "data"})
-        self.client_add("sc/foo/bar")
-        self.client_commit("sc", "second message")
+        dc = self.commit_editor()
+        foo = dc.open_dir("foo")
+        foo.add_file("foo/bar").modify("data")
+        dc.close()
 
         svnbranch = self.svndir.open_branch()
         bzrbranch = self.bzrdir.open_branch()
@@ -69,11 +151,12 @@ class TestPush(TestCaseWithSubversionRepository):
         self.assertEqual(0, result.new_revno - result.old_revno)
 
     def test_diverged(self):
-        self.build_tree({'sc/foo/bar': "data"})
-        self.client_add("sc/foo/bar")
-        self.client_commit("sc", "second message")
+        dc = self.commit_editor()
+        foo = dc.open_dir("foo")
+        foo.add_file("foo/bar").modify("data")
+        dc.close()
 
-        svndir = BzrDir.open("sc")
+        svndir = BzrDir.open(self.repos_url)
 
         self.build_tree({'dc/file': 'data'})
         wt = self.bzrdir.open_workingtree()
@@ -92,7 +175,7 @@ class TestPush(TestCaseWithSubversionRepository):
         svnbranch = self.svndir.open_branch()
         svnbranch.pull(self.bzrdir.open_branch())
 
-        repos = self.svndir._find_repository()
+        repos = self.svndir.find_repository()
         mapping = repos.get_mapping()
         self.assertEquals(newid, svnbranch.last_revision())
         inv = repos.get_inventory(repos.generate_revision_id(2, "", mapping))
@@ -113,7 +196,7 @@ class TestPush(TestCaseWithSubversionRepository):
 
         self.svndir.open_branch().pull(self.bzrdir.open_branch())
 
-        repos = self.svndir._find_repository()
+        repos = self.svndir.find_repository()
         mapping = repos.get_mapping()
         inv = repos.get_inventory(repos.generate_revision_id(2, "", mapping))
         self.assertTrue(inv.has_filename('file'))
@@ -123,7 +206,7 @@ class TestPush(TestCaseWithSubversionRepository):
                         self.svndir.open_branch().last_revision())
 
     def test_override_revprops(self):
-        self.svndir._find_repository().get_config().set_user_option("override-svn-revprops", "True")
+        self.svndir.find_repository().get_config().set_user_option("override-svn-revprops", "True")
         self.build_tree({'dc/file': 'data'})
         wt = self.bzrdir.open_workingtree()
         wt.add('file')
@@ -132,7 +215,7 @@ class TestPush(TestCaseWithSubversionRepository):
         self.svndir.open_branch().pull(self.bzrdir.open_branch())
 
         self.assertEquals(("Sombody famous", "2002-02-01T23:00:00.000000Z", "Commit from Bzr"), 
-            self.client_log(self.repos_url)[2][1:])
+            self.client_log(self.repos_url, 0, 2)[2][1:])
 
     def test_empty_file(self):
         self.build_tree({'dc/file': ''})
@@ -142,7 +225,7 @@ class TestPush(TestCaseWithSubversionRepository):
 
         self.svndir.open_branch().pull(self.bzrdir.open_branch())
 
-        repos = self.svndir._find_repository()
+        repos = self.svndir.find_repository()
         mapping = repos.get_mapping()
         inv = repos.get_inventory(repos.generate_revision_id(2, "", mapping))
         self.assertTrue(inv.has_filename('file'))
@@ -162,7 +245,7 @@ class TestPush(TestCaseWithSubversionRepository):
 
         self.svndir.open_branch().pull(self.bzrdir.open_branch())
 
-        repos = self.svndir._find_repository()
+        repos = self.svndir.find_repository()
         mapping = repos.get_mapping() 
         inv = repos.get_inventory(repos.generate_revision_id(2, "", mapping))
         self.assertTrue(inv.has_filename('south'))
@@ -177,7 +260,7 @@ class TestPush(TestCaseWithSubversionRepository):
 
         self.svndir.open_branch().pull(self.bzrdir.open_branch())
 
-        repos = self.svndir._find_repository()
+        repos = self.svndir.find_repository()
         mapping = repos.get_mapping()
         inv = repos.get_inventory(repos.generate_revision_id(2, "", mapping))
         self.assertTrue(inv.has_filename('file'))
@@ -205,6 +288,35 @@ class TestPush(TestCaseWithSubversionRepository):
         self.assertEqual(Branch.open("dc").revision_history(), 
                          Branch.open("b").revision_history())
 
+    def test_fetch_preserves_file_revid(self):
+        self.build_tree({'dc/file': 'data'})
+        wt = self.bzrdir.open_workingtree()
+        wt.add('file')
+        self.build_tree({'dc/foo/bla': 'data43243242'})
+        revid = wt.commit(message="Commit from Bzr")
+
+        self.svndir.open_branch().pull(self.bzrdir.open_branch())
+
+        os.mkdir("b")
+        repos = self.svndir.sprout("b")
+
+        b = Branch.open("b")
+
+        def check_tree_revids(rtree):
+            self.assertEqual(rtree.inventory[rtree.path2id("file")].revision,
+                             revid)
+            self.assertEqual(rtree.inventory[rtree.path2id("foo")].revision,
+                             b.revision_history()[1])
+            self.assertEqual(rtree.inventory[rtree.path2id("foo/bla")].revision,
+                             revid)
+            self.assertEqual(rtree.inventory.root.revision, b.revision_history()[0])
+
+        check_tree_revids(wt.branch.repository.revision_tree(b.last_revision()))
+
+        check_tree_revids(b.repository.revision_tree(b.last_revision()))
+        bc = self.svndir.open_branch()
+        check_tree_revids(bc.repository.revision_tree(bc.last_revision()))
+
     def test_message(self):
         self.build_tree({'dc/file': 'data'})
         wt = self.bzrdir.open_workingtree()
@@ -213,7 +325,7 @@ class TestPush(TestCaseWithSubversionRepository):
 
         self.svndir.open_branch().pull(self.bzrdir.open_branch())
 
-        repos = self.svndir._find_repository()
+        repos = self.svndir.find_repository()
         mapping = repos.get_mapping()
         self.assertEqual("Commit from Bzr",
           repos.get_revision(repos.generate_revision_id(2, "", mapping)).message)
@@ -226,9 +338,9 @@ class TestPush(TestCaseWithSubversionRepository):
 
         self.svndir.open_branch().pull(self.bzrdir.open_branch())
 
-        self.client_update("sc")
+        c = ra.RemoteAccess(self.repos_url)
         self.assertEqual("3 some-rid\n", 
-                self.client_get_prop("sc", SVN_PROP_BZR_REVISION_ID+"none"))
+                c.get_dir("", c.get_latest_revnum())[2][SVN_PROP_BZR_REVISION_ID+"none"])
 
     def test_commit_check_rev_equal(self):
         self.build_tree({'dc/file': 'data'})
@@ -238,7 +350,7 @@ class TestPush(TestCaseWithSubversionRepository):
 
         self.svndir.open_branch().pull(self.bzrdir.open_branch())
 
-        rev1 = self.svndir._find_repository().get_revision(wt.branch.last_revision())
+        rev1 = self.svndir.find_repository().get_revision(wt.branch.last_revision())
         rev2 = self.bzrdir.find_repository().get_revision(wt.branch.last_revision())
 
         self.assertEqual(rev1.committer, rev2.committer)
@@ -260,7 +372,7 @@ class TestPush(TestCaseWithSubversionRepository):
 
         self.svndir.open_branch().pull(self.bzrdir.open_branch())
 
-        repos = self.svndir._find_repository()
+        repos = self.svndir.find_repository()
 
         mapping = repos.get_mapping()
 
@@ -285,17 +397,17 @@ class TestPush(TestCaseWithSubversionRepository):
                 repos.get_ancestry(wt.branch.last_revision()))
 
     def test_multiple_diverged(self):
-        oc_url = self.make_client("o", "oc")
+        oc_url = self.make_repository("o")
 
         self.build_tree({'dc/file': 'data'})
         wt = self.bzrdir.open_workingtree()
         wt.add('file')
         wt.commit(message="Commit from Bzr")
 
-        self.build_tree({'oc/file': 'data2', 'oc/adir': None})
-        self.client_add("oc/file")
-        self.client_add("oc/adir")
-        self.client_commit("oc", "Another commit from Bzr")
+        oc = self.get_commit_editor(oc_url)
+        oc.add_file("file").modify("data2")
+        oc.add_dir("adir")
+        oc.close()
 
         self.assertRaises(DivergedBranches, 
                 lambda: Branch.open(oc_url).pull(self.bzrdir.open_branch()))
@@ -303,35 +415,40 @@ class TestPush(TestCaseWithSubversionRepository):
     def test_different_branch_path(self):
         # A       ,> C
         # \ -> B /
-        self.build_tree({'sc/trunk/foo': "data", 'sc/branches': None})
-        self.client_add("sc/trunk")
-        self.client_add("sc/branches")
-        self.client_commit("sc", "foo")
+        dc = self.commit_editor()
+        trunk = dc.add_dir("trunk")
+        trunk.add_file('trunk/foo').modify("data")
+        dc.add_dir("branches")
+        dc.close()
 
-        self.client_copy('sc/trunk', 'sc/branches/mybranch')
-        self.build_tree({'sc/branches/mybranch/foo': "data2"})
-        self.client_commit("sc", "add branch")
+        dc = self.commit_editor()
+        branches = dc.open_dir('branches')
+        mybranch = branches.add_dir('branches/mybranch', 'trunk')
+        mybranch.open_file("branches/mybranch/foo").modify('data2')
+        dc.close()
 
-        self.svndir = BzrDir.open("sc/branches/mybranch")
+        self.svndir = BzrDir.open("%s/branches/mybranch" % self.repos_url)
         os.mkdir("mybranch")
         self.bzrdir = self.svndir.sprout("mybranch")
 
         self.build_tree({'mybranch/foo': 'bladata'})
         wt = self.bzrdir.open_workingtree()
         revid = wt.commit(message="Commit from Bzr")
-        push(Branch.open("sc/trunk"), wt.branch, 
+        b = Branch.open("%s/trunk" % self.repos_url)
+        push(b.repository.get_graph(), b, wt.branch.repository, 
              wt.branch.revision_history()[-2])
-        mutter('log %r' % self.client_log("sc/trunk")[4][0])
+        mutter('log %r' % self.client_log("%s/trunk" % self.repos_url, 0, 4)[4][0])
         self.assertEquals('M',
-            self.client_log("sc/trunk")[4][0]['/trunk'].action)
-        push(Branch.open("sc/trunk"), wt.branch, wt.branch.last_revision())
-        mutter('log %r' % self.client_log("sc/trunk")[5][0])
+            self.client_log("%s/trunk" % self.repos_url, 0, 4)[4][0]['/trunk'][0])
+        b = Branch.open("%s/trunk" % self.repos_url)
+        push(b.repository.get_graph(), b, wt.branch.repository, wt.branch.last_revision())
+        mutter('log %r' % self.client_log("%s/trunk" % self.repos_url, 0, 5)[5][0])
         self.assertEquals("/branches/mybranch", 
-            self.client_log("sc/trunk")[5][0]['/trunk'].copyfrom_path)
+            self.client_log("%s/trunk" % self.repos_url, 0, 5)[5][0]['/trunk'][1])
 
 class PushNewBranchTests(TestCaseWithSubversionRepository):
     def test_single_revision(self):
-        repos_url = self.make_client("a", "dc")
+        repos_url = self.make_repository("a")
         bzrwt = BzrDir.create_standalone_workingtree("c", 
             format=format.get_rich_root_format())
         self.build_tree({'c/test': "Tour"})
@@ -348,15 +465,18 @@ class PushNewBranchTests(TestCaseWithSubversionRepository):
         self.assertEquals([revid], newbranch.revision_history())
 
     def test_single_revision_single_branching_scheme(self):
-        repos_url = self.make_client("a", "dc")
+        repos_url = self.make_repository("a")
         bzrwt = BzrDir.create_standalone_workingtree("c", 
             format=format.get_rich_root_format())
         self.build_tree({'c/test': "Tour"})
         bzrwt.add("test")
         revid = bzrwt.commit("Do a commit")
-        self.build_tree({"dc/some/funny/branch": None})
-        self.client_add("dc/some")
-        self.client_commit("dc", "msg")
+
+        dc = self.get_commit_editor(repos_url)
+        some = dc.add_dir("some")
+        funny = some.add_dir("some/funny")
+        funny.add_dir("some/funny/branch")
+        dc.close()
         newdir = BzrDir.open("%s/some/funny/branch/name" % repos_url)
         newbranch = newdir.import_branch(bzrwt.branch)
         self.assertEquals(revid, newbranch.last_revision())
@@ -457,8 +577,59 @@ class PushNewBranchTests(TestCaseWithSubversionRepository):
         self.assertTrue(os.path.exists("bzrco/baz.txt"))
         wt.branch.push(Branch.open(repos_url+"/trunk"))
 
+    def test_push_merge_unchanged_file(self):
+        def check_tree(t):
+            self.assertEquals(base_revid, t.inventory[t.path2id("bar.txt")].revision)
+            self.assertEquals(other_revid, t.inventory[t.path2id("bar2.txt")].revision)
+        repos_url = self.make_repository("test")
+
+        dc = self.get_commit_editor(repos_url)
+        trunk = dc.add_dir("trunk")
+        trunk.add_file("trunk/foo.txt").modify("add file")
+        dc.close()
+
+        os.mkdir('bzrco1')
+        dir1 = BzrDir.open(repos_url+"/trunk").sprout("bzrco1")
+
+        os.mkdir('bzrco2')
+        dir2 = BzrDir.open(repos_url+"/trunk").sprout("bzrco2")
+
+        wt1 = dir1.open_workingtree()
+        self.build_tree({'bzrco1/bar.txt': 'bar'})
+        wt1.add("bar.txt")
+        base_revid = wt1.commit("add another file", rev_id="mybase")
+        wt1.branch.push(Branch.open(repos_url+"/trunk"))
+
+        wt2 = dir2.open_workingtree()
+        self.build_tree({'bzrco2/bar2.txt': 'bar'})
+        wt2.add("bar2.txt")
+        other_revid = wt2.commit("add yet another file", rev_id="side1")
+
+        wt1.lock_write()
+        try:
+            wt1.merge_from_branch(wt2.branch)
+            self.assertEquals([wt1.last_revision(), other_revid], wt1.get_parent_ids())
+            mergingrevid = wt1.commit("merge", rev_id="side2")
+            check_tree(wt1.branch.repository.revision_tree(mergingrevid))
+        finally:
+            wt1.unlock()
+        self.assertTrue(os.path.exists("bzrco1/bar2.txt"))
+        wt1.branch.push(Branch.open(repos_url+"/trunk"))
+        r = Repository.open(repos_url)
+        props = r.branchprop_list.get_changed_properties("trunk", 3)
+        self.assertEquals(props['bzr:text-parents'], 'bar2.txt\tside1\n')
+
+        os.mkdir("cpy")
+        cpy = BzrDir.create("cpy", format.get_rich_root_format())
+        cpyrepos = cpy.create_repository()
+        r.copy_content_into(cpyrepos)
+        check_tree(cpyrepos.revision_tree(mergingrevid))
+
+        t = r.revision_tree(mergingrevid)
+        check_tree(t)
+
     def test_missing_prefix_error(self):
-        repos_url = self.make_client("a", "dc")
+        repos_url = self.make_repository("a")
         bzrwt = BzrDir.create_standalone_workingtree("c", 
             format=format.get_rich_root_format())
         self.build_tree({'c/test': "Tour"})
@@ -469,7 +640,7 @@ class PushNewBranchTests(TestCaseWithSubversionRepository):
                           lambda: newdir.import_branch(bzrwt.branch))
 
     def test_repeat(self):
-        repos_url = self.make_client("a", "dc")
+        repos_url = self.make_repository("a")
         bzrwt = BzrDir.create_standalone_workingtree("c", 
             format=format.get_rich_root_format())
         self.build_tree({'c/test': "Tour"})
@@ -486,7 +657,7 @@ class PushNewBranchTests(TestCaseWithSubversionRepository):
                           bzrwt.branch)
 
     def test_multiple(self):
-        repos_url = self.make_client("a", "dc")
+        repos_url = self.make_repository("a")
         bzrwt = BzrDir.create_standalone_workingtree("c", 
             format=format.get_rich_root_format())
         self.build_tree({'c/test': "Tour"})
@@ -500,7 +671,7 @@ class PushNewBranchTests(TestCaseWithSubversionRepository):
         self.assertEquals([revid1, revid2], newbranch.revision_history())
 
     def test_dato(self):
-        repos_url = self.make_client("a", "dc")
+        repos_url = self.make_repository("a")
         bzrwt = BzrDir.create_standalone_workingtree("c", 
             format=format.get_rich_root_format())
         self.build_tree({'c/foo.txt': "foo"})
@@ -513,7 +684,7 @@ class PushNewBranchTests(TestCaseWithSubversionRepository):
                 Repository.open(repos_url).get_revision(revid1).committer)
 
     def test_utf8_commit_msg(self):
-        repos_url = self.make_client("a", "dc")
+        repos_url = self.make_repository("a")
         bzrwt = BzrDir.create_standalone_workingtree("c", 
             format=format.get_rich_root_format())
         self.build_tree({'c/foo.txt': "foo"})
@@ -525,11 +696,14 @@ class PushNewBranchTests(TestCaseWithSubversionRepository):
                 Repository.open(repos_url).get_revision(revid1).message)
 
     def test_multiple_part_exists(self):
-        repos_url = self.make_client("a", "dc")
-        self.build_tree({'dc/trunk/myfile': "data", 'dc/branches': None})
-        self.client_add('dc/trunk')
-        self.client_add('dc/branches')
-        self.client_commit("dc", "Message")
+        repos_url = self.make_repository("a")
+
+        dc = self.get_commit_editor(repos_url)
+        trunk = dc.add_dir('trunk')
+        trunk.add_file("trunk/myfile").modify("data")
+        dc.add_dir("branches")
+        dc.close()
+
         svnrepos = Repository.open(repos_url)
         os.mkdir("c")
         bzrdir = BzrDir.open(repos_url+"/trunk").sprout("c")
@@ -547,10 +721,12 @@ class PushNewBranchTests(TestCaseWithSubversionRepository):
             , revid1, revid2], newbranch.revision_history())
 
     def test_push_overwrite(self):
-        repos_url = self.make_client("a", "dc")
-        self.build_tree({'dc/trunk/bloe': "text"})
-        self.client_add("dc/trunk")
-        self.client_commit("dc", "initial")
+        repos_url = self.make_repository("a")
+
+        dc = self.get_commit_editor(repos_url)
+        trunk = dc.add_dir("trunk")
+        trunk.add_file("trunk/bloe").modify("text")
+        dc.close()
 
         os.mkdir("d1")
         bzrdir = BzrDir.open(repos_url+"/trunk").sprout("d1")
@@ -566,19 +742,53 @@ class PushNewBranchTests(TestCaseWithSubversionRepository):
 
         self.build_tree({'d2/myfile': "France"})
         bzrwt2.add("myfile")
-        revid2 = bzrwt1.commit("Do a commit")
+        revid2 = bzrwt2.commit("Do a commit")
 
         bzrwt1.branch.push(Branch.open(repos_url+"/trunk"))
-
-        raise KnownFailure("push --overwrite not supported yet")
+        self.assertEquals(bzrwt1.branch.revision_history(),
+                Branch.open(repos_url+"/trunk").revision_history())
 
         bzrwt2.branch.push(Branch.open(repos_url+"/trunk"), overwrite=True)
 
-        self.assertEquals([revid2], 
+        self.assertEquals(bzrwt2.branch.revision_history(),
                 Branch.open(repos_url+"/trunk").revision_history())
 
+    def test_push_overwrite_unrelated(self):
+        repos_url = self.make_repository("a")
+
+        dc = self.get_commit_editor(repos_url)
+        trunk = dc.add_dir("trunk")
+        trunk.add_file("trunk/bloe").modify("text")
+        dc.close()
+
+        os.mkdir("d1")
+        bzrdir = BzrDir.open(repos_url+"/trunk").sprout("d1")
+        bzrwt1 = bzrdir.open_workingtree()
+
+        bzrwt2 = BzrDir.create_standalone_workingtree("d2", 
+            format=format.get_rich_root_format())
+
+        self.build_tree({'d1/myfile': "Tour"})
+        bzrwt1.add("myfile")
+        revid1 = bzrwt1.commit("Do a commit")
+
+        self.build_tree({'d2/myfile': "France"})
+        bzrwt2.add("myfile")
+        revid2 = bzrwt2.commit("Do a commit")
+
+        bzrwt1.branch.push(Branch.open(repos_url+"/trunk"))
+        self.assertEquals(bzrwt1.branch.revision_history(),
+                Branch.open(repos_url+"/trunk").revision_history())
+
+        bzrwt2.branch.push(Branch.open(repos_url+"/trunk"), overwrite=True)
+
+        self.assertEquals(bzrwt2.branch.revision_history(),
+                Branch.open(repos_url+"/trunk").revision_history())
+
+
+
     def test_complex_rename(self):
-        repos_url = self.make_client("a", "dc")
+        repos_url = self.make_repository("a")
         bzrwt = BzrDir.create_standalone_workingtree("c", 
             format=format.get_rich_root_format())
         self.build_tree({'c/registry/generic.c': "Tour"})
@@ -608,7 +818,7 @@ class PushNewBranchTests(TestCaseWithSubversionRepository):
         BzrDir.open(repos_url+"/trunk").sprout("n")
 
     def test_rename_dir_changing_contents(self):
-        repos_url = self.make_client("a", "dc")
+        repos_url = self.make_repository("a")
         bzrwt = BzrDir.create_standalone_workingtree("c", 
             format=format.get_rich_root_format())
         self.build_tree({'c/registry/generic.c': "Tour"})
@@ -636,7 +846,7 @@ class PushNewBranchTests(TestCaseWithSubversionRepository):
         check(copybranch)
     
     def test_rename_dir(self):
-        repos_url = self.make_client("a", "dc")
+        repos_url = self.make_repository("a")
         bzrwt = BzrDir.create_standalone_workingtree("c", 
             format=format.get_rich_root_format())
         self.build_tree({'c/registry/generic.c': "Tour"})
@@ -662,7 +872,7 @@ class PushNewBranchTests(TestCaseWithSubversionRepository):
         from bzrlib.debug import debug_flags
         debug_flags.add("commit")
         debug_flags.add("fetch")
-        repos_url = self.make_client("a", "dc")
+        repos_url = self.make_repository("a")
         bzrwt = BzrDir.create_standalone_workingtree("c", 
             format=format.get_rich_root_format())
         self.build_tree({'c/registry/generic.c': "Tour"})
@@ -677,16 +887,21 @@ class PushNewBranchTests(TestCaseWithSubversionRepository):
         newbranch = newdir.import_branch(bzrwt.branch)
 
         # Should create dc/trunk
-        self.client_update("dc")
 
-        self.build_tree({'dc/branches': None})
-        self.client_add("dc/branches")
-        self.client_copy("dc/trunk", "dc/branches/foo")
-        self.client_commit("dc", "Copy branches")
-        self.client_update("dc")
+        dc = self.get_commit_editor(repos_url)
+        branches = dc.add_dir("branches")
+        branches.add_dir('branches/foo', 'trunk')
+        dc.close()
 
-        self.build_tree({'dc/branches/foo/registry/generic.c': "France"})
-        merge_revno = self.client_commit("dc", "Change copied branch")[0]
+        dc = self.get_commit_editor(repos_url)
+        branches = dc.open_dir("branches")
+        foo = branches.open_dir("branches/foo")
+        registry = foo.open_dir("branches/foo/registry")
+        registry.open_file("branches/foo/registry/generic.c").modify("France")
+        dc.close()
+
+        r = ra.RemoteAccess(repos_url)
+        merge_revno = r.get_latest_revnum()
         merge_revid = newdir.find_repository().generate_revision_id(merge_revno, "branches/foo", mapping)
 
         self.build_tree({'c/registry/generic.c': "de"})
@@ -701,13 +916,12 @@ class PushNewBranchTests(TestCaseWithSubversionRepository):
         trunk.pull(bzrwt.branch)
 
         self.assertEquals([revid1, revid2, revid3], trunk.revision_history())
-        self.client_update("dc")
         self.assertEquals(
                 '1 initialrevid\n2 changerevid\n3 mergerevid\n',
-                self.client_get_prop("dc/branches/foo", SVN_PROP_BZR_REVISION_ID+"trunk0"))
+                self.client_get_prop(repos_url+"/branches/foo", SVN_PROP_BZR_REVISION_ID+"trunk0", r.get_latest_revnum()))
 
     def test_complex_replace_dir(self):
-        repos_url = self.make_client("a", "dc")
+        repos_url = self.make_repository("a")
         bzrwt = BzrDir.create_standalone_workingtree("c", 
             format=format.get_rich_root_format())
         self.build_tree({'c/registry/generic.c': "Tour"})
@@ -733,7 +947,7 @@ class PushNewBranchTests(TestCaseWithSubversionRepository):
         from bzrlib.debug import debug_flags
         debug_flags.add("commit")
         debug_flags.add("fetch")
-        repos_url = self.make_client("a", "dc")
+        repos_url = self.make_repository("a")
         bzrwt = BzrDir.create_standalone_workingtree("c", 
             format=format.get_rich_root_format())
         self.build_tree({'c/registry/generic.c': "Tour"})
@@ -746,15 +960,14 @@ class PushNewBranchTests(TestCaseWithSubversionRepository):
         newdir = BzrDir.open(repos_url+"/trunk")
         newbranch = newdir.import_branch(bzrwt.branch)
 
-        # Should create dc/trunk
-        self.client_update("dc")
+        c = ra.RemoteAccess(repos_url)
+        self.assertTrue(c.check_path("trunk/registry/generic.c", c.get_latest_revnum()) == core.NODE_FILE)
 
-        self.assertTrue(os.path.exists("dc/trunk/registry/generic.c"))
-        sleep(1) # Subversion relies on timestamps to detect 
-                 # changed files...
-        self.build_tree({'dc/trunk/registry/generic.c': "BLA"})
-        self.client_commit("dc/trunk", "Change copied branch")
-        self.client_update("dc")
+        dc = self.get_commit_editor(repos_url)
+        trunk = dc.open_dir("trunk")
+        registry = trunk.open_dir("trunk/registry")
+        registry.open_file("trunk/registry/generic.c").modify("BLA")
+        dc.close()
         mapping = newdir.find_repository().get_mapping()
         merge_revid = newdir.find_repository().generate_revision_id(2, "trunk", mapping)
 
@@ -771,9 +984,43 @@ class PushNewBranchTests(TestCaseWithSubversionRepository):
                 trunk.repository.get_revision(revid2).parent_ids)
 
         self.assertEquals([revid1, revid2], trunk.revision_history())
-        self.client_update("dc")
         self.assertEquals(
                 '1 initialrevid\n2 mergerevid\n',
-                self.client_get_prop("dc/trunk", SVN_PROP_BZR_REVISION_ID+"trunk0"))
+                self.client_get_prop(repos_url+"/trunk", SVN_PROP_BZR_REVISION_ID+"trunk0",
+                                     c.get_latest_revnum()))
 
 
+class TestPushTwice(TestCaseWithSubversionRepository):
+    def test_push_twice(self):
+        # bug 208566
+        repos_url = self.make_repository('d')
+
+        dc = self.get_commit_editor(repos_url)
+        trunk = dc.add_dir("trunk")
+        foo = trunk.add_dir("trunk/foo")
+        foo.add_file("trunk/foo/bla").modify("data")
+        dc.add_dir("branches")
+        dc.close()
+
+        svndir = BzrDir.open(repos_url+"/trunk")
+        os.mkdir("dc")
+        bzrdir = svndir.sprout("dc")
+        wt = bzrdir.open_workingtree()
+        revid = wt.commit(message="Commit from Bzr")
+        expected_history = wt.branch.revision_history()
+
+        svndir1 = BzrDir.open(repos_url+"/branches/a")
+        svndir1.import_branch(wt.branch)
+        self.assertEquals(expected_history, svndir1.open_branch().revision_history())
+
+        svndir2 = BzrDir.open(repos_url+"/branches/b")
+        svndir2.import_branch(wt.branch)
+        self.assertEquals(expected_history, svndir2.open_branch().revision_history())
+
+        revid = wt.commit(message="Commit from Bzr")
+        expected_history = wt.branch.revision_history()
+
+        svndir1.open_branch().pull(wt.branch)
+        self.assertEquals(expected_history, svndir1.open_branch().revision_history())
+        svndir2.open_branch().pull(wt.branch)
+        self.assertEquals(expected_history, svndir2.open_branch().revision_history())
