@@ -19,139 +19,158 @@ from bzrlib.inventory import Inventory, TreeReference
 from bzrlib.osutils import has_symlinks
 from bzrlib.repository import Repository
 from bzrlib.revision import NULL_REVISION
-from bzrlib.tests import TestCase
+from bzrlib.tests import TestSkipped
 from bzrlib.workingtree import WorkingTree
 
-import errors
-import os
-from tree import (SvnBasisTree, parse_externals_description, 
-                  inventory_add_external)
-import sys
-from tests import TestCaseWithSubversionRepository
+from bzrlib.plugins.svn import errors
+from bzrlib.plugins.svn.core import SubversionException
+from bzrlib.plugins.svn.tests import TestCaseWithSubversionRepository
+from bzrlib.plugins.svn.tree import SvnBasisTree, inventory_add_external
+
 
 class TestBasisTree(TestCaseWithSubversionRepository):
     def test_executable(self):
-        self.make_client("d", "dc")
-        self.build_tree({"dc/file": "x"})
-        self.client_add("dc/file")
-        self.client_set_prop("dc/file", "svn:executable", "*")
-        self.client_commit("dc", "executable")
-        tree = SvnBasisTree(self.open_checkout("dc"))
+        repos_url = self.make_client("d", "dc")
+
+        dc = self.get_commit_editor(repos_url)
+        f = dc.add_file("file")
+        f.modify("x")
+        f.change_prop("svn:executable", "*")
+        dc.close()
+
+        self.client_update("dc")
+
+        tree = SvnBasisTree(WorkingTree.open("dc"))
         self.assertTrue(tree.inventory[tree.inventory.path2id("file")].executable)
 
     def test_executable_changed(self):
-        self.make_client("d", "dc")
-        self.build_tree({"dc/file": "x"})
-        self.client_add("dc/file")
-        self.client_commit("dc", "executable")
+        repos_url = self.make_client("d", "dc")
+
+        dc = self.get_commit_editor(repos_url)
+        dc.add_file("file").modify("x")
+        dc.close()
+
         self.client_update("dc")
         self.client_set_prop("dc/file", "svn:executable", "*")
-        tree = SvnBasisTree(self.open_checkout("dc"))
+        tree = SvnBasisTree(WorkingTree.open("dc"))
         self.assertFalse(tree.inventory[tree.inventory.path2id("file")].executable)
 
     def test_symlink(self):
-        if not has_symlinks():
-            return
-        self.make_client("d", "dc")
-        os.symlink("target", "dc/file")
-        self.build_tree({"dc/file": "x"})
-        self.client_add("dc/file")
-        self.client_commit("dc", "symlink")
+        repos_url = self.make_client("d", "dc")
+
+        dc = self.get_commit_editor(repos_url)
+        file = dc.add_file("file")
+        file.modify("link target")
+        file.change_prop("svn:special", "*")
+        dc.close()
+
         self.client_update("dc")
-        tree = SvnBasisTree(self.open_checkout("dc"))
+        tree = SvnBasisTree(WorkingTree.open("dc"))
         self.assertEqual('symlink', 
                          tree.inventory[tree.inventory.path2id("file")].kind)
         self.assertEqual("target",
                          tree.inventory[tree.inventory.path2id("file")].symlink_target)
 
-    def test_symlink_next(self):
-        if not has_symlinks():
-            return
-        self.make_client("d", "dc")
-        os.symlink("target", "dc/file")
-        self.build_tree({"dc/file": "x", "dc/bla": "p"})
-        self.client_add("dc/file")
-        self.client_add("dc/bla")
-        self.client_commit("dc", "symlink")
-        self.build_tree({"dc/bla": "pa"})
-        self.client_commit("dc", "change")
+    def test_symlink_with_newlines_in_target(self):
+        repos_url = self.make_client("d", "dc")
+
+        dc = self.get_commit_editor(repos_url)
+        file = dc.add_file("file")
+        file.modify("link target\nbar\nbla")
+        file.change_prop("svn:special", "*")
+        dc.close()
+
         self.client_update("dc")
-        tree = SvnBasisTree(self.open_checkout("dc"))
+        tree = SvnBasisTree(WorkingTree.open("dc"))
+        self.assertEqual('symlink', 
+                         tree.inventory[tree.inventory.path2id("file")].kind)
+        self.assertEqual("target\nbar\nbla",
+                         tree.inventory[tree.inventory.path2id("file")].symlink_target)
+
+    def test_symlink_not_special(self):
+        repos_url = self.make_client("d", "dc")
+
+        dc = self.get_commit_editor(repos_url)
+        file = dc.add_file("file")
+        file.modify("fsdfdslhfdsk h")
+        file.change_prop("svn:special", "*")
+        file2 = dc.add_file("file2")
+        file.modify("a")
+        file.change_prop("svn:special", "*")
+        dc.close()
+
+        try:
+            self.client_update("dc")
+        except SubversionException, (msg, num):
+            if num == errors.ERR_WC_BAD_ADM_LOG:
+                raise TestSkipped("Unable to run test with svn 1.4")
+            raise
+        tree = SvnBasisTree(WorkingTree.open("dc"))
+        self.assertEqual('file', 
+                         tree.inventory[tree.inventory.path2id("file")].kind)
+
+    def test_symlink_next(self):
+        repos_url = self.make_client("d", "dc")
+
+        dc = self.get_commit_editor(repos_url)
+        dc.add_file("bla").modify("p")
+        file = dc.add_file("file")
+        file.modify("link target")
+        file.change_prop("svn:special", "*")
+        dc.close()
+
+        dc = self.get_commit_editor(repos_url)
+        dc.open_file("bla").modify("pa")
+        dc.close()
+
+        self.client_update("dc")
+
+        tree = SvnBasisTree(WorkingTree.open("dc"))
         self.assertEqual('symlink', 
                          tree.inventory[tree.inventory.path2id("file")].kind)
         self.assertEqual("target",
                          tree.inventory[tree.inventory.path2id("file")].symlink_target)
 
+    def test_annotate_iter(self):
+        repos_url = self.make_client("d", "dc")
+
+        dc = self.get_commit_editor(repos_url)
+        dc.add_file("file").modify("x\n")
+        dc.close()
+
+        dc = self.get_commit_editor(repos_url)
+        dc.open_file("file").modify("x\ny\n")
+        dc.close()
+
+        self.client_update('dc')
+        tree = SvnBasisTree(WorkingTree.open("dc"))
+        self.assertRaises(NotImplementedError, tree.annotate_iter, tree.path2id("file"))
+
     def test_executable_link(self):
         if not has_symlinks():
             return
-        self.make_client("d", "dc")
-        os.symlink("target", "dc/file")
-        self.build_tree({"dc/file": "x"})
-        self.client_add("dc/file")
-        self.client_set_prop("dc/file", "svn:executable", "*")
-        self.client_commit("dc", "exe1")
-        wt = self.open_checkout("dc")
+        repos_url = self.make_client("d", "dc")
+
+        dc = self.get_commit_editor(repos_url)
+        file = dc.add_file("file")
+        file.modify("link target")
+        file.change_prop("svn:special", "*")
+        file.change_prop("svn:executable", "*")
+        dc.close()
+
+        try:
+            self.client_update("dc")
+        except SubversionException, (msg, num):
+            if num == errors.ERR_WC_BAD_ADM_LOG:
+                raise TestSkipped("Unable to run test with svn 1.4")
+            raise
+
+        wt = WorkingTree.open("dc")
         tree = SvnBasisTree(wt)
         self.assertFalse(tree.inventory[tree.inventory.path2id("file")].executable)
         self.assertFalse(wt.inventory[wt.inventory.path2id("file")].executable)
 
 
-class TestExternalsParser(TestCase):
-    def test_parse_root_relative_externals(self):
-        self.assertRaises(NotImplementedError, parse_externals_description, 
-                    "http://example.com", "third-party/skins              ^/foo")
-
-    def test_parse_scheme_relative_externals(self):
-        self.assertRaises(NotImplementedError, parse_externals_description, 
-                    "http://example.com", "third-party/skins              //foo")
-
-    def test_parse_externals(self):
-        self.assertEqual({
-            'third-party/sounds': (None, "http://sounds.red-bean.com/repos"),
-            'third-party/skins': (None, "http://skins.red-bean.com/repositories/skinproj"),
-            'third-party/skins/toolkit': (21, "http://svn.red-bean.com/repos/skin-maker")},
-            parse_externals_description("http://example.com",
-"""third-party/sounds             http://sounds.red-bean.com/repos
-third-party/skins              http://skins.red-bean.com/repositories/skinproj
-third-party/skins/toolkit -r21 http://svn.red-bean.com/repos/skin-maker"""))
-
-    def test_parse_comment(self):
-        self.assertEqual({
-            'third-party/sounds': (None, "http://sounds.red-bean.com/repos")
-                },
-            parse_externals_description("http://example.com/",
-"""
-
-third-party/sounds             http://sounds.red-bean.com/repos
-#third-party/skins              http://skins.red-bean.com/repositories/skinproj
-#third-party/skins/toolkit -r21 http://svn.red-bean.com/repos/skin-maker"""))
-
-    def test_parse_relative(self):
-        self.assertEqual({
-            'third-party/sounds': (None, "http://example.com/branches/other"),
-                },
-            parse_externals_description("http://example.com/trunk",
-"third-party/sounds             ../branches/other"))
-
-    def test_parse_repos_root_relative(self):
-        self.assertEqual({
-            'third-party/sounds': (None, "http://example.com/bar/bla/branches/other"),
-                },
-            parse_externals_description("http://example.com/trunk",
-"third-party/sounds             /bar/bla/branches/other"))
-
-    def test_parse_invalid_missing_url(self):
-        """No URL specified."""
-        self.assertRaises(errors.InvalidExternalsDescription, 
-            lambda: parse_externals_description("http://example.com/", "bla"))
-            
-    def test_parse_invalid_too_much_data(self):
-        """No URL specified."""
-        self.assertRaises(errors.InvalidExternalsDescription, 
-            lambda: parse_externals_description(None, "bla -R40 http://bla/"))
- 
-
 class TestInventoryExternals(TestCaseWithSubversionRepository):
     def test_add_nested_norev(self):
         """Add a nested tree with no specific revision referenced."""
